<?php
namespace Paymentpage\Admin;

use Paymentpage\includes;

/**
 * The admin-specific functionality of the plugin.
 *
 * @link https://brandonfire.com/
 * @since 1.0.0
 *       
 * @package Payment_Page
 * @subpackage Payment_Page/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package Payment_Page
 * @subpackage Payment_Page/admin
 * @author Brandonfire <brandon@brandonfire.com>
 */
class Admin
{

    /**
     * The ID of this plugin.
     *
     * @since 1.0.0
     * @access private
     * @var string $plugin_name The ID of this plugin.
     */
    private static $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since 1.0.0
     * @access private
     * @var string $version The current version of this plugin.
     */
    private static $version;

    private static $Auth;

    private static $Stripe;

    private static $Payments_Gateway;

    private static $Freemius;

    private static $Setup;

    /**
     * Initialize the class and set its properties.
     *
     * @since 1.0.0
     * @param string $plugin_name
     *            The name of this plugin.
     * @param string $version
     *            The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {
        self::$plugin_name = $plugin_name;
        self::$version = $version;
        self::$Setup = new \Paymentpage\includes\controllers\SetupController();
        self::$Auth = new \Paymentpage\includes\controllers\AuthController();
        self::$Payments_Gateway = new \Paymentpage\includes\controllers\PaymentsGatewayControllers();
        self::$Stripe = new \Paymentpage\includes\controllers\StripeController();
        self::$Freemius = new \Paymentpage\includes\controllers\FreemiusController();

        add_action('admin_init', array(
            $this,
            'Setup'
        ));
        add_action('pp_admin_enqueue_scripts', array(
            $this,
            'enqueue_scripts'
        ));
    }

    public function admin_bar_item_stripe( $admin_bar )
    {   
        $stripe_mode = self::$Stripe->get_stripe_mode();
        
        $admin_bar->add_menu( array(
            'id'     => 'payment-page-connection-status',
            'href'   => admin_url( 'admin.php?page=payment-page' ),
            'parent' => 'top-secondary',
            'title'  => __( 'Payment Page ' . ucfirst( $stripe_mode ) . ' Mode', 'payment-page' ),
            'meta'   => array(
            'class' => $stripe_mode,
        ),
        ) );
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since 1.0
     */
    public function enqueue_styles()
    {}

    /**
     * Register the JavaScript for the admin area.
     *
     * @since 1.0
     */
    public static function enqueue_scripts()
    {
        wp_register_style('react-setup-style', plugin_dir_url(__FILE__) . 'css/main.css', true);
        wp_enqueue_style('payment-page-dashboard', plugin_dir_url(__FILE__) . 'css/dashboard.css');

        wp_register_script("react-chunk", plugin_dir_url(__FILE__) . 'js/react/react.js', array(
            'jquery'
        ), self::$version, true);
        
        wp_register_script("react-setup", plugin_dir_url(__FILE__) . 'js/react/main.js', array(
            'jquery'
        ), self::$version, true);

        $user_info = wp_get_current_user();
        
        wp_localize_script("react-setup", 'wpr_object', array(
            'api_nonce' => wp_create_nonce('wp_rest'),
            'PUBLIC_URL' => '/wp-content/plugins/' . PAYMENT_PAGE_DIR_NAME . '/public',
            'UPGRADE_URL' => self::$Freemius->get_upgrade_url(),
            'USER_EMAIL' => $user_info->user_email
        ));
    }


    public function register_settings(){
        add_settings_section(
            'general_settings_section',         // ID used to identify this section and with which to register options
            'Sandbox Options',                  // Title to be displayed on the administration page
            array($this, 'toggle_stripe_mode_description'), // Callback used to render the description of the section
            'general'                           // Page on which to add this section of options
        );
        add_settings_field( 
            'is_on_test_mode',                      // ID used to identify the field throughout the theme
            'Is On Test Mode',                           // The label to the left of the option interface element
            array($this,'toggle_stripe_mode'),   // The name of the function responsible for rendering the option interface
            'general',                          // The page on which this option will be displayed
            'general_settings_section',         // The name of the section to which this field belongs
            array(                              // The array of arguments to pass to the callback. In this case, just a description.
                'Activate this setting to use stripe on test mode.'
            )
        );

        register_setting(
            'general',
            'is_on_test_mode'
        );
    }

    public function toggle_stripe_mode($args) {
        $html = '<input type="checkbox" id="is_on_test_mode" name="is_on_test_mode" value="1" ' . checked(1, get_option('is_on_test_mode'), false) . '/>';          
        $html .= '<label for="is_on_test_mode"> '  . $args[0] . '</label>'; 
        echo $html;
         
    } // end toggle_stripe_mode

    public function toggle_stripe_mode_description() {
        echo '<p>Select which areas of content you wish to display.</p>';
    } 

    /**
     *
     * @action:Handle Admin Page view
     * This functions takes the request page and will redirect the user to the payment page setup if is not installed yet
     * TODO: get the option that validate the setup process was finished.
     * IE: (SETUP_FINISHED, TRUE)
     * @since:1.0
     */
    public static function Setup()
    {
        $Auth = self::$Auth;
        $Stripe = self::$Stripe;
        $Payments_Gateway = self::$Payments_Gateway;
        if (! isset($_GET['page']) && empty($_GET['page']))
            return;

        $page = sanitize_text_field($_GET['page']);
        // TODO: display setup process or dashboard for the next version. Now try to redirect everytime to setup process.

        // Stripe Connect will redirect to the WP instance with that Requests params. It's a signal that the login
        // was successfully.
        if ('payment-page' === $page) {
            if (isset($_GET['stripe_user_id']) && isset($_GET['ajax_token'])) {

                $mode = ( isset($_GET['mode']) ? sanitize_text_field($_GET['mode'] ) : 'test'  );
                $ajax_token = sanitize_text_field($_GET['ajax_token']);
                $user_account_id = sanitize_text_field($_GET['stripe_user_id']);
                $currencies = (isset($_GET['currencies']) && ! empty($_GET['currencies']) ? sanitize_text_field($_GET['currencies']) : array());
                
                if (empty($ajax_token) || empty($ajax_token))
                    return false;

                $valid_token = $Auth->is_valid_token($ajax_token);
                if (! empty($currencies)) {
                    $Stripe->set_currencies_available(explode(',', $currencies));
                }

                // Ok, token is valid. Continue
                if ($valid_token && isset($_GET['gateway']) && ! empty($_GET['gateway'])) {
                    $gateway = sanitize_text_field($_GET['gateway']);
                    switch ($gateway) {
                        case "stripe":
                            // We support only Stripe. So, fowarding to step 2 weh success stripe user id was installed

                            if ($Stripe->setup_user_account_id($user_account_id, $mode)) {
                                self::$Payments_Gateway->set_gateways_selected('stripe');
                            }
                            break;
                    }
                    
                    wp_redirect("admin.php?page=payment-page&success=true&gateway=stripe");
                    exit;

                    
                } else {
                    // TODO: handle error. System cannot find wich gateway configure
                }
                // TODO: we need to move it inside if "payment-page-setup" conditional after create our settings page
            }
        }
    }
}