<?php
namespace Paymentpage\includes;

use Paymentpage\includes;

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link https://brandonfire.com/
 * @since 1.0.0
 *       
 * @package Payment_Page
 * @subpackage Payment_Page/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since 1.0.0
 * @package Payment_Page
 * @subpackage Payment_Page/includes
 * @author Brandonfire <brandon@brandonfire.com>
 */
class Main
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since 1.0.0
     * @access protected
     * @var Payment_Page_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since 1.0.0
     * @access protected
     * @var string $plugin_name The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since 1.0.0
     * @access protected
     * @var string $version The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        if (defined('PAYMENT_PAGE_VERSION')) {
            $this->version = PAYMENT_PAGE_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'payment-page';
        $this->loader = new Loader();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since 1.0.0
     * @access private
     */
    private function define_admin_hooks()
    {
        $plugin_admin = new \Paymentpage\admin\Admin($this->get_plugin_name(), $this->get_version());
        $plugin_api = new PaymentPageApi();
        $elementor = new \Paymentpage\includes\elements\Widgets();

        Loader::add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        Loader::add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');
        Loader::add_action('rest_api_init', $plugin_api, 'init_api');
        Loader::add_action('init', $plugin_admin, 'Setup');
        Loader::add_action( 'admin_bar_menu', $plugin_admin, 'admin_bar_item_stripe' );
        //Loader::add_action('admin_init', $plugin_admin, 'register_settings'); // PRM 2020-12-18 Disabled; only used for test switch

    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since 1.0.0
     * @access private
     */
    private function define_public_hooks()
    {
        $plugin_public = new \Paymentpage\includes\PaymentPagePublic($this->get_plugin_name(), $this->get_version());
        Loader::add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        Loader::add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since 1.0.0
     */
    public static function run()
    {
        Loader::run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since 1.0.0
     * @return string The name of the plugin.
     */
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since 1.0.0
     * @return Payment_Page_Loader Orchestrates the hooks of the plugin.
     */
    public function get_loader()
    {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since 1.0.0
     * @return string The version number of the plugin.
     */
    public function get_version()
    {
        return $this->version;
    }
}