<?php
namespace Paymentpage\includes\controllers;

// use \Stripe_Controller as Stripe;
class PaymentsGatewayControllers
{

    const PP_GATEWAYS_PREFERED = "PP_GATEWAYS_PREFERED";
    private static $Stripe;

    public function __construct()
    {
        self::$Stripe = new  \Paymentpage\includes\controllers\StripeController();
        self::install_payment_gateways();
    }

    /**
     * Install all paymentgatewas
     * TODO: find a way to get unique gateways from the json.
     */
    public static function install_payment_gateways()
    {
        $payment_gateways = array(
            array(
                "name" => "stripe",
                "available" => true
            ),
            array(
                "name" => "square",
                "available" => false
            ),
            array(
                "name" => "securepay",
                "available" => false
            ),
            array(
                "name" => "worldpay",
                "available" => false
            ),
            array(
                "name" => "authorize",
                "available" => false
            ),
            array(
                "name" => "2checkout",
                "available" => false
            ),
            array(
                "name" => "paypal",
                "available" => false
            ),
            array(
                "name" => "braintree",
                "available" => false
            )
        );
        update_option(PAYMENT_PAGE_GATEWAYS_OPTION, $payment_gateways);
    }

    /**
     * Get all availables gateways
     * TODO: return from DB stored and activated paymentgateways by the plugin.
     */
    public static function get_gateways()
    {
        $gateways = get_option(PAYMENT_PAGE_GATEWAYS_OPTION);

        $payment_gateways = array();        
        // Set metadata for each gateway
        foreach ($gateways as $gateway) {

            if ($gateway['name'] === 'stripe') {
                $stripe = self::$Stripe->get_config();
                $payment_gateways[] = $stripe;
            } else {
                $payment_gateways[] = $gateway;
            }
        }
        return $payment_gateways;
    }

    /**
     * Get gateways selected by user
     *
     * @return false | array()
     */
    public static function get_gateways_selected()
    {
        $gateways_selected = get_option(PAYMENT_PAGE_GATEWAYS_SELECTED);
        return $gateways_selected;
    }

    /**
     * Update all the content for gateway selected option at once
     *
     * @param $gatewas [id]
     */
    public static function bulk_update_gateways_selected($gateways)
    {
        if (is_array($gateways)) {
            update_option(PAYMENT_PAGE_GATEWAYS_SELECTED, $gateways);
        }
    }

    /**
     * Set a new gateway selected
     *
     * @param $gateway [id]
     */
    public static function set_gateways_selected($gateway)
    {
        $gateways_selected = self::get_gateways_selected();

        // Option could not be initialized yet, create instead.
        // We want to be an array. Intialize as it.
        if ($gateways_selected === false) {
            add_option(PAYMENT_PAGE_GATEWAYS_SELECTED, array(
                $gateway
            ));
        }

        // Option could not be initialized yet, check if is an array
        if (is_array($gateways_selected) && ! in_array($gateway, $gateways_selected)) {

            array_push($gateways_selected, $gateway);
            update_option(PAYMENT_PAGE_GATEWAYS_SELECTED, $gateways_selected);
        }
    }

    /**
     * Save selected methods by the user into the DB
     */
    public static function set_methods_options_selected($gateway, $methods)
    {
        if ($gateway === 'stripe') {
            self::$Stripe->save_payment_methods_selected($methods);
        } else {
            return 'Not other methods available yet.';
        }
    }

    /**
     * Remove a gateway from selected gateways
     *
     * @param $gateway [id]
     */
    public static function remove_gateway_selected($gateway)
    {
        $gateways_selected = get_option(PAYMENT_PAGE_GATEWAYS_SELECTED);
        if (array_search($gateway, $gateways_selected) !== false) {
            $updated_gateways_selected = array_diff($gateways_selected, array(
                $gateway
            ));
            update_option(PAYMENT_PAGE_GATEWAYS_SELECTED, $updated_gateways_selected);
        }
    }

    /**
     * Save prefered gateways selected by the user.
     * Those gateways would not be available yet.
     * Save it let us to send a notification when one of them are available.
     *
     *  @params $gateways array( gatewayID: number )
     */
    public static function set_prefered_non_avaiable_gateways($gateway)
    {
        // Prevent save non array
        $gateways = get_option(self::PP_GATEWAYS_PREFERED);
        $gateways[] = $gateway;
        return update_option(self::PP_GATEWAYS_PREFERED, $gateways);
    }

    public static function get_prefered_non_avaiable_gateways()
    {
        $gateways = get_option(self::PP_GATEWAYS_PREFERED);        
        if (! $gateways)
            return array();
        return $gateways;
    }

    public static function get_payment_methods_by_gateway_name($gateway_name)
    {
        if ($gateway_name === "stripe")
            return self::$Stripe->get_payment_methods();
    }

    /**
     * Set gateway notification
     * When a gateways is not available yet, let user to be notified when it's ready to use
     * TODO: set a label on the selected gateway
     */
    public static function set_gateway_notification($gateway)
    {}
}