<?php
namespace Paymentpage\includes\controllers;

class SetupController
{

    const PP_FIRST_PAYMENT_PAGE_DATA = "PP_FIRST_PAYMENT_PAGE_DATA";

    const PP_TEMPLATE_SELECTED = "PP_TEMPLATE_SELECTED";

    const PP_OPTIN_DATA = "PP_OPTIN_DATA";

    private static $Payments_Gateway;

    private static $Templates_Controller;

    public function __construct()
    {
        self::$Payments_Gateway = new \Paymentpage\includes\controllers\PaymentsGatewayControllers();
        self::$Templates_Controller = new \Paymentpage\includes\controllers\TemplateController();
    }

    /**
     * Start Setup
     * Executed every time the user are into the setup page.
     * Return previous state of existing installation and the whole information to this process
     *
     * @since 1.0
     * @return [ paymentgatewas, templates, step ]
     */
    public static function start_setup()
    {
        $setup_data = array();
        // Gateways
        $setup_data['paymentGateways'] = self::$Payments_Gateway->get_gateways();

        // Gateway selected
        $setup_data['paymentGatewaysSelected'] = self::$Payments_Gateway->get_gateways_selected();

        // Gateway Waiting List
        $setup_data['waitingGatewaysList'] = self::$Payments_Gateway->get_prefered_non_avaiable_gateways();

        // Templates
        $setup_data['templates'] = self::$Templates_Controller->get_templates();
        $setup_data['template_selected'] = self::get_template_selected();

        // First Payment Page Data Saved
        $setup_data['firstProductPageData'] = self::get_first_payment_page_data();

        $setup_data['step'] = self::get_setup_step();

        // Will be removed on the next release of the component
        $setup_data['wp_nonce'] = wp_create_nonce('ajax-nonce');
        // Newsletters options
        return $setup_data;
    }

    /**
     * Create a draft page with template selected on setup step
     * TODO: validate if not first payment page was created.
     *
     * @since 1.0
     *        Return the created id instead of create a new draft page.
     */
    public static function create_draft_page($template_selected_id)
    {
        global $wpdb;

        $api_templates = "https://payment.page/wp-json/wp/v2/pages/$template_selected_id";

        try {
            $response = wp_remote_get($api_templates);

            if (is_wp_error($response)) {
                return array(
                    "error" => "Connection error"
                );
            }

            $remote_template = json_decode($response['body']);

            if (! isset($remote_template)) {
                return 1;
            }

            // Let's add a proper handler error here in the future
 
            if (! isset($remote_template->metadata)) {
                if (isset($remote_template->data->status)) {
                    if ($remote_template->data->status == 404) {
                        return array(
                            "error" => $remote_template->data->status
                        );
                    }
                } else {
                    return array(
                        "error" => "No data"
                    );
                }
            }

            $content = $remote_template->content->rendered;

            $post_meta_infos = $remote_template->metadata;

            $post_details = array(
                'post_title' => 'First payment page',
                'post_content' => $content,
                'post_status' => 'draft',
                'post_author' => 1,
                'post_type' => 'page'
            );
            $draft_page_id = wp_insert_post($post_details);

            if (count((array) $post_meta_infos) != 0) {
                $sql_query = "INSERT INTO $wpdb->postmeta (post_id, meta_key, meta_value) ";
                foreach ($post_meta_infos as $_meta_key => $meta_info) {
                    $meta_key = $_meta_key;
                    if ($meta_key == '_wp_old_slug')
                        continue;
                    $meta_value = addslashes($meta_info[0]);
                    $sql_query_sel[] = "SELECT $draft_page_id, '$meta_key', '$meta_value'";
                }
                $sql_query .= implode(" UNION ALL ", $sql_query_sel);
                $wpdb->query($sql_query);
            }
            return $draft_page_id;
        } catch (Exception $e) {
            return array(
                "error" => $e
            );
        }
    }

    /**
     * Save opt-in data
     *
     * @since 1.0
     */
    public static function save_optin_data($data)
    {
        return update_option(PP_OPTIN_DATA, $data);
    }

    /**
     * Set Setup step
     *
     * @sice 1.0
     * @param $step number
     */
    public static function set_setup_step($step)
    {
        return update_option(PAYMENT_PAGE_SETUP_STEP_NUMBER, $step);
    }

    /**
     * Get Setup step
     *
     * @sice 1.0
     * @return false | $step (number)
     */
    public static function get_setup_step()
    {
        $step = get_option(PAYMENT_PAGE_SETUP_STEP_NUMBER);
        if ($step == 0 || $step == false)
            return 1;
        return (int) $step;
    }

    /**
     * Temporall function.
     * I think we'll need more complex data store if we want to let user to get
     * history data
     *
     *  @sice 1.0
     */
    public static function save_first_payment_page_data($data)
    {
        return update_option(self::PP_FIRST_PAYMENT_PAGE_DATA, $data);
    }

    public static function get_first_payment_page_data()
    {
        return get_option(self::PP_FIRST_PAYMENT_PAGE_DATA);
    }

    public static function save_template_selected($template_id)
    {
        if (empty($template_id))
            return;
        return update_option(self::PP_TEMPLATE_SELECTED, $template_id);
    }

    public static function get_template_selected()
    {
        return get_option(self::PP_TEMPLATE_SELECTED);
    }
}