<?php
namespace Paymentpage\includes\controllers;

// namespace Stripe_Controller;
class StripeController
{

    private const PP_STRIPE_USER_ID_TEST = "PP_STRIPE_USER_ID_TEST";
    private const PP_STRIPE_USER_ID_LIVE = "PP_STRIPE_USER_ID_LIVE";

    private const PP_STRIPE_ENABLED = "PP_STRIPE_ENABLED";

    private const PP_STRIPE_MODE = "PP_STRIPE_MODE";

    private const PP_STRIPE_SELECTED_METHODS = "PP_STRIPE_SELECTED_METHODS";

    private const PP_STRIPE_CURRENCIES_AVAILABLE = "PP_STRIPE_CURRENCIES_AVAILABLE";

    private static $Auth;

    public function __construct()
    {
        self::$Auth = new \Paymentpage\includes\controllers\AuthController();
    }

    /**
     * Get Auth Link for Setup front-end step 1
     */
    private static function get_auth_link($mode)
    {
        $auth_link = "";
        $actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

        update_option('payment_page_url', $actual_link);

        // $token_gen = rand(99, mt_getrandmax());
        // update_option('payment_page_connect_token', $token_gen);
        $token = \Paymentpage\includes\controllers\AuthController::get_user_token();
        if (! $token)
            $token = self::$Auth->generate_user_token();
        $protocols = array(
            'http://',
            'http://www.',
            'www.'
        );
        $domain_url = str_replace($protocols, '', get_bloginfo('wpurl'));
        // PRM 2020-12-18 Disabled; only used for test switch
        $mode = (int) ($mode === 'live' ? true : false);
        $auth_link = "https://api.payment.page/stripeconnect/callstripeconnect/" . $mode . "/live/" . $token . "/ajaxtoken/" . $domain_url . "/website/setup/page";
        return $auth_link;
    }

    /**
     * Get stored token for the user
     * TODO: save token into an option or custom table
     */
    private static function get_account_id($mode)
    {
        return self::get_stripe_user_account_id($mode);
    }

    public static function set_stripe_mode( $mode ){
        return (boolean) update_option(self::PP_STRIPE_MODE, $mode);
    }

    public static function get_stripe_mode(){
        if( !get_option(self::PP_STRIPE_MODE) ){
             self::set_stripe_mode('test');
        }
        return get_option(self::PP_STRIPE_MODE);
    }

    public static function get_config()
    {


        $connection_state = array(
            'live' => array(
                'auth_link' => self::get_auth_link('live'),
                'account_id' => self::get_account_id('live'),
            ),
            'test' =>  array(
                'auth_link' => self::get_auth_link('test'),
                'account_id' => self::get_account_id('test')
            ),
        );

        $stripe_config = array();
        // TODO: will get from plugin option
        $stripe_config['name'] = 'stripe';
        $stripe_config['available'] = true;
        $stripe_config['enabled'] = self::is_enabled();
        $stripe_config['payment_methods'] = self::get_payment_methods();
        $stripe_config['currencies'] = self::get_currencies_available();
        $stripe_config['connection_state'] = $connection_state;
        $stripe_config['mode'] = self::get_stripe_mode();
        return $stripe_config;
    }

    /**
     * Get all available payment methods
     * TODO: get it from a real API
     */
    public static function get_payment_methods()
    {
        $payment_options = array();
        $methods = array(
            array(
                "name" => "visa"
            ),
            array(
                "name" => "wechatpay"
            ),
            array(
                "name" => "alipay"
            ),
            array(
                "name" => "ach"
            ),
            array(
                "name" => "applepay"
            ),
            array(
                "name" => "googlepay"
            ),
            array(
                "name" => "fpx"
            ),
            array(
                "name" => "giropay"
            ),
            array(
                "name" => "bancontact"
            ),
            array(
                "name" => "ideal"
            ),
            array(
                "name" => "sepa"
            ),
            array(
                "name" => "sofort"
            ),
            array(
                "name" => "mb"
            ),
            array(
                "name" => "przelewy"
            ),
            array(
                "name" => "klarna"
            ),
            array(
                "name" => "eps"
            )
        );

        // Check if user has selected methods
        $selected_methods = self::get_payment_methods_selected();
        if (count($methods) > 0) {
            foreach ($methods as $method) {
                $selected = false;
                foreach ($selected_methods as $selected_method) {
                    if ($selected_method['name'] === $method['name'])
                        $selected = true;
                }

                $payment_options[] = array(
                    'name' => $method['name'],
                    'selected' => $selected
                );
            }
        }

        return $payment_options;
    }

    private static function set_enabled($option)
    {
        // For some reason user try to login again. Delete the option in order to let the WP update option return true;
        update_option(self::PP_STRIPE_ENABLED, $option);
        return;
    }

    public static function is_enabled()
    {
        return (boolean) get_option(self::PP_STRIPE_ENABLED);
    }



    /**
     * Save selected method
     */
    public function save_payment_methods_selected($methods)
    {
        return update_option(self::PP_STRIPE_SELECTED_METHODS, $methods);
    }

    public static function get_payment_methods_selected()
    {
        $selected_method = get_option(self::PP_STRIPE_SELECTED_METHODS);

        if ($selected_method === false) {
            return array();
        }
        return get_option(self::PP_STRIPE_SELECTED_METHODS);
    }


    public static function disconnect_account($mode){
        return (boolean) delete_option( $mode === 'live' ? self::PP_STRIPE_USER_ID_LIVE : self::PP_STRIPE_USER_ID_TEST );
    }

    /**
     * Store user id into the option and set this gateway enabled to use.
     */
    public function setup_user_account_id($user_id, $mode)
    {
        // For some reason user try to login again. Delete the option in order to let the WP update option return true;
        delete_option( $mode === 'live' ? self::PP_STRIPE_USER_ID_LIVE : self::PP_STRIPE_USER_ID_TEST );
        self::set_enabled(true);
        return update_option( $mode === 'live' ? self::PP_STRIPE_USER_ID_LIVE : self::PP_STRIPE_USER_ID_TEST, $user_id);
    }

    public static function get_stripe_user_account_id($mode)
    {
        return get_option( $mode === 'live' ? self::PP_STRIPE_USER_ID_LIVE : self::PP_STRIPE_USER_ID_TEST );
    }

    public function set_currencies_available($currencies)
    {
        return update_option(self::PP_STRIPE_CURRENCIES_AVAILABLE, $currencies);
    }

    public static function delete_stripe_data(){
        delete_option(self::PP_STRIPE_CURRENCIES_AVAILABLE);
        delete_option(self::PP_STRIPE_USER_ID_LIVE);
        delete_option(self::PP_STRIPE_USER_ID_TEST);
        delete_option(self::PP_STRIPE_MODE);
        delete_option(PAYMENT_PAGE_GATEWAYS_OPTION);
    }

    public static function get_currencies_available()
    {
        return get_option(self::PP_STRIPE_CURRENCIES_AVAILABLE);
    }

    public static function create_stripe_product($data){
        // $env = getenv(WP_ENV);
        // $domain = $env === 'local' ? 'http://local.api-payment-page.com' : 'https://api.payment.page'; 
        $domain = 'https://api.payment.page';
        $url = "$domain/stripeconnect/create_product";
        // TODO: get URL for payment intent TEST AND SANDBOX;
        $requests_response = wp_remote_post($url, [
            'headers' => [
                'Content-Type' => 'application/json;'
            ],
            'body' => json_encode($data),
            'timeout' => 10
        ]);
        return wp_remote_retrieve_body($requests_response);

    }

    public static function create_stripe_customer($data){
        // $env = getenv(WP_ENV);
        // $domain = $env === 'local' ? 'http://local.api-payment-page.com' : 'https://api.payment.page';
        
        $domain = 'https://api.payment.page';
        
        $url = "$domain/stripeconnect/create_customer";
        // TODO: get URL for payment intent TEST AND SANDBOX;
        $requests_response = wp_remote_post($url, [
            'headers' => [
                'Content-Type' => 'application/json;'
            ],
            'body' => json_encode($data),
            'timeout' => 10
        ]);
        return wp_remote_retrieve_body($requests_response);

    }

    public static function create_payment_intent($payment_intent)
    {
        // $env = getenv(WP_ENV);
        // $domain = $env === 'local' ? 'http://local.api-payment-page.com' : 'https://api.payment.page';
        
        $domain = 'https://api.payment.page';

        $url = "$domain/stripeconnect/payment_intent";
        // TODO: get URL for payment intent TEST AND SANDBOX;
        $requests_response = wp_remote_post($url, [
            'headers' => [
                'Content-Type' => 'application/json;'
            ],
            'body' => json_encode($payment_intent),
            'timeout' => 10
        ]);
        return wp_remote_retrieve_body($requests_response);
    }

    public static function create_subscription($data)
    {
        // $env = getenv(WP_ENV);
        // $domain = $env === 'local' ? 'http://local.api-payment-page.com' : 'https://api.payment.page';
       
        $domain = 'https://api.payment.page';

        $url = "$domain/stripeconnect/create_subscription";
        // TODO: get URL for payment intent TEST AND SANDBOX;
        $requests_response = wp_remote_post($url, [
            'headers' => [
                'Content-Type' => 'application/json;'
            ],
            'body' => json_encode($data),
            'timeout' => 10
        ]);
        return wp_remote_retrieve_body($requests_response);
    }

}
