<?php

namespace Paymentpage\includes\elements\Controlls\Pricing;

/**
 * Pricing emoji one area control.
 *
 * A control for displaying a pricing repeater for plans
 *
 * @since 1.0.0
 */
use  Elementor\Modules\DynamicTags\Module as TagsModule ;
class PricingControll extends \Elementor\Control_Base_Multiple
{
    private static  $freemius ;
    public function __construct( $stripe_config )
    {
        global  $payment_page_fs ;
        self::$freemius = $payment_page_fs;
        $this->stripe_config = $stripe_config;
        $this->frequencies = array(
            0 => array(
            'value' => 'one-time',
            'label' => 'One-time',
        ),
            1 => array(
            'value' => 'd_1',
            'label' => 'Daily',
        ),
            2 => array(
            'value' => 'w_1',
            'label' => 'Weekly',
        ),
            3 => array(
            'value' => 'm_1',
            'label' => 'Monthly',
        ),
            4 => array(
            'value' => 'm_4',
            'label' => 'Quarterly',
        ),
            5 => array(
            'value' => 'm_6',
            'label' => 'Every 6 months',
        ),
            6 => array(
            'value' => 'y_1',
            'label' => 'Annually',
        ),
            7 => array(
            'value' => 'y_3',
            'label' => 'Every 3 years',
        ),
        );
        $this->currencyVariations = array();
        if ( $this->stripe_config['currencies'] ) {
            foreach ( $this->stripe_config['currencies'] as $currency ) {
                foreach ( $this->frequencies as $frequency ) {
                    $this->currencyVariations[$currency]['frequencies'][] = $frequency;
                    $this->currencyVariations[$currency]['usedFrequencies'] = [];
                }
            }
        }
    }
    
    /**
     * Retrieve the control type, in this case `emojionearea`.
     *
     * @since 1.0.0
     * @access public
     *        
     * @return string Control type.
     */
    public function get_type()
    {
        return 'pricingplans';
    }
    
    /**
     *
     * Used to register and enqueue custom scripts and styles used by the emoji one
     * area control.
     *
     * @since 1.0.0
     * @access public
     */
    public function enqueue()
    {
        $scripts_vars = array();
        $scripts_vars['FREE_VERSION_MODE'] = ( self::$freemius->is_free_plan() ? 1 : 0 );
        $scripts_vars['currencies'] = $this->stripe_config['currencies'];
        wp_enqueue_style(
            'pricing-control-style',
            plugins_url( '/assets/css/pricing-control.css', __FILE__ ),
            [],
            '3.4.1'
        );
        wp_register_script(
            'pricing-control',
            plugins_url( '/assets/js/pricing-control.js', __FILE__ ),
            [ 'jquery' ],
            '1.2.0'
        );
        wp_enqueue_script( 'pricing-control' );
        wp_localize_script( 'pricing-control', 'stripe_data', $scripts_vars );
    }
    
    public function get_default_value()
    {
        $premium_default_values = array();
        $free_default_values = array();
        $free_default_values = array( array(
            "price"     => 99,
            "currency"  => "usd",
            "frequency" => array(
            'value' => 'one-time',
            'label' => 'One-time',
        ),
        ) );
        if ( self::$freemius->is_free_plan() ) {
            return $free_default_values;
        }
    }
    
    public function get_value( $control, $settings )
    {
        if ( !isset( $control['default'] ) ) {
            $control['default'] = $this->get_default_value();
        }
        
        if ( isset( $settings[$control['name']] ) ) {
            $value = $settings[$control['name']];
        } else {
            $value = $control['default'];
        }
        
        return $value;
    }
    
    /**
     *
     * Used to generate the control HTML in the editor using Underscore JS
     * template. The variables for the class are available using `data` JS
     * object.
     *
     * @since 1.0.0
     * @access public
     */
    public function content_template()
    {
        $control_uid = $this->get_control_uid();
        ?>
<div class="elementor-control-field pricing-plan-repeater">
	<label for="<?php 
        echo  esc_attr( $control_uid ) ;
        ?>"
		class="elementor-control-title">{{{ data.label }}}</label>
	<div class="elementor-control-input-wrapper">
        <input data-setting="currencyVariationsStore" id="<?php 
        echo  $this->get_control_uid( 'currencyVariationsStore' ) ;
        ?>" type="hidden" value="<?php 
        echo  htmlentities( json_encode( $this->currencyVariations ) ) ;
        ?>"></input>
		<div class="fields-repeater-container">
			<div class="field-group">
				<div>
						<?php 
        $control_uid_price = $this->get_control_uid( 'price' );
        ?>

							<label> Price <input id="<?php 
        echo  $control_uid_price ;
        ?>"
						data-setting="price" class="field price-input" type="number">
					</label>
				</div>
				<div>
						<?php 
        $control_uid_currency = $this->get_control_uid( 'currency' );
        ?>
							<label> Currency <select
						id="<?php 
        echo  esc_attr( $control_uid_currency ) ;
        ?>"
						data-setting="currency" class="field currency-selector">
							<option value="usd">USD</option>
							<option value="eur">EUR</option>
					</select>
					</label>
				</div>
                <?php 
        ?>
			</div>
		</div>
                <?php 
        ?>
			</div>
</div>
<# if ( data.description ) { #>
<div class="elementor-control-field-description">{{{ data.description
	}}}</div>
<# } #>
		<?php 
    }

}