(function ($) {
  $(window).load(function () {
    var pricingplansItemView = window.elementor.modules.controls.BaseData.extend(
      {
        // Set presaved values
        onReady: function () {
          var pricings = this.getControlValue();
          this.clearRow();
          this.FREE_VERSION_MODE = stripe_data.FREE_VERSION_MODE;
          this.currencies = stripe_data.currencies;
          this.frequencies = stripe_data.frequencies;
          this.variationStore = this.initVariationStore();
          this.defaultFrequencyObj = {
            value: "one-time",
            label: "One-time",
          };

          this.defaultPricingObj = {
            currency: "usd", // get default currency account
            frequency: this.defaultFrequencyObj,
            price: pricings[0].price,
          };

          var self = this;
          if (this.FREE_VERSION_MODE > 0) {
            pricings = new Array(this.defaultPricingObj);
          }
          // Iterate over every group of field saved.
          _.each(pricings, function (rowData, rowIndex) {
            self.appendNewRow(rowData, rowIndex);
          });
        },
        initVariationStore: function () {
          var self = this;

          getFrequencyByValue = function (frequenvyValue) {
            if (self.FREE_VERSION_MODE <= 0) {
              return self.frequencies.filter(
                (frequency) => frequency.value === frequenvyValue
              )[0];
            } else {
              return this.defaultFrequencyObj;
            }
          };

          var getVariations = function () {
            return JSON.parse(jQuery(self.ui.currencyVariationsStore).val());
          };

          var getVariationsByCurrency = function (currency) {
            var variations = getVariations();
            return variations[currency].frequencies;
          };

          var getVariationsUsedByCurrency = function (currency) {
            var variations = getVariations();
            var usedFrequencies = variations[currency].usedFrequencies;
            return usedFrequencies;
          };

          var getNonUsedVariationsUsedByCurrency = function (currency) {
            var variations = getVariations();
            var noUsed = variations[currency].frequencies.filter(function (
              frequency
            ) {
              return (
                variations[currency].usedFrequencies.findIndex(function (used) {
                  return used.label === frequency.label;
                }) === -1
              );
            });
            return noUsed;
          };

          var addFrequency = function (currency, frequencyToAdd) {
            if (!currency) return;
            var variations = getVariations();
            var fequency = getFrequencyByValue(frequencyToAdd);
            variations[currency].usedFrequencies = [
              ...getVariationsUsedByCurrency(currency),
              { ...fequency },
            ];
            updateStore(variations);
          };

          var isFrequencyAvailableForCurrency = function (currency, frequency) {
            var variationsUsed = getVariationsUsedByCurrency(currency);
            if (variationsUsed.length <= 0) return true;
            return (
              variationsUsed.findIndex(function (variation) {
                if (variation.value === frequency) return variation;
              }) === -1
            );
          };

          var addFrequencyToAvailable = function (currency, frequencyToAdd) {
            if (!currency) return;
            var variations = getVariations();
            var frequency = getFrequencyByValue(frequencyToAdd);
            variations[currency].usedFrequencies = _.without(
              variations[currency].usedFrequencies,
              _.findWhere(variations[currency].usedFrequencies, {
                value: frequency.value,
              })
            );
            updateStore(variations);
          };

          var updateAvailableFrequencyByCurrency = function (
            currency,
            frequencyUsed
          ) {
            var variations = getVariations();
            let usedFrequencies = variations[currency].frequencies.filter(
              function (frequency) {
                if (frequency.value == frequencyUsed.value) return true;
              }
            );
            variations[currency].usedFrequencies = usedFrequencies;
            updateStore(variations);
          };

          var updateStore = function (newVal) {
            jQuery(self.ui.currencyVariationsStore).val(JSON.stringify(newVal));
          };

          var getCurrenciesByFrequency = function (frequencySelected) {
            return self.currencies.filter(function (currency) {
              var variations = getVariationsUsedByCurrency(currency);
              return (
                variations.findIndex(function (used) {
                  return used.value === frequencySelected;
                }) === -1
              );
            });
          };

          /**
           * Get options available for this frequency selector based on currency selected
           * @param {*} field
           * @param {*} currency
           */
          var refreshFrequencyFieldOptions = function (field, currency) {
            var $selectField = jQuery(field);
            var selectedfrequencyObj = getFrequencyByValue($selectField.val());
            $selectField.empty();
            var variations = getNonUsedVariationsUsedByCurrency(currency);
            jQuery.each(variations, function (key, frequency) {
              $selectField.append(
                $("<option>").val(frequency.value).text(frequency.label)
              );
            });

            if (selectedfrequencyObj) {
              $selectField.append(
                $("<option>")
                  .val(selectedfrequencyObj.value)
                  .text(selectedfrequencyObj.label)
              );
              $selectField.val(selectedfrequencyObj.value);
            } else {
              $selectField.val("");
            }
          };

          var refreshCurrencyFieldOptions = function (field, frequency) {
            var $selectField = jQuery(field);
            var selectedCurrency = $selectField.val();
            $selectField.empty();
            var currencies = getCurrenciesByFrequency(frequency);
            jQuery.each(currencies, function (key, currency) {
              $selectField.append($("<option>").val(currency).text(currency));
            });

            if (selectedCurrency) {
              $selectField.val(selectedCurrency);
            } else {
              $selectField.val("");
            }
          };

          return {
            getVariationsByCurrency,
            getNonUsedVariationsUsedByCurrency,
            getVariations,
            updateAvailableFrequencyByCurrency,
            addFrequency,
            addFrequencyToAvailable,
            refreshFrequencyFieldOptions,
            updateStore,
            getFrequencyByValue,
            refreshCurrencyFieldOptions,
            isFrequencyAvailableForCurrency,
          };
        },
        ui: function ui() {
          return {
            inputPrice: 'input[data-setting="price"]',
            currencyVariationsStore:
              'input[data-setting="currencyVariationsStore"]',
            selectCurrency: 'select[data-setting="currency"]',
            selectFrequency: 'select[data-setting="frequency"]',
            btnRemoveRow: ".remove-price-row",
            btnAddRow: ".add-more-options",
            repeaterContainer: ".fields-repeater-container",
          };
        },
        events: function events() {
          return {
            "keyup @ui.inputPrice": "onFieldChange",
            "change @ui.selectFrequency": "changedField",
            "click @ui.selectFrequency": "prevOptionSelected",
            "change @ui.selectCurrency": "changedField",
            "click @ui.selectCurrency": "prevOptionSelected",
            "click @ui.btnAddRow": "onButtonAddRowClick",
            "click @ui.btnRemoveRow": "onButtonRemoveRowClick",
          };
        },
        prevOptionSelected: function (event) {
          var currentFieldType = $(event.currentTarget).attr("data-setting"); // frequency | currency;
          var currentRow = $(event.currentTarget).closest(".field-group");

          // TODO: make this as a function to avoid repeat code
          if (currentFieldType === "frequency") {
            var currentFrequencySelector = $(currentRow).find(
              ".frequency-selector"
            );
            var currentFrequency = jQuery(currentFrequencySelector).val();
            currentFrequencySelector.attr("prev", currentFrequency);
          }

          if (currentFieldType === "currency") {
            var currencySelector = $(currentRow).find(".currency-selector");
            var currencyVal = jQuery(currencySelector).val();
            currencySelector.attr("prev", currencyVal);
          }
        },
        onButtonAddRowClick: function onButtonAddRowClick() {
          this.appendNewRow(false);
        },
        changedField: function (event) {
          var self = this;
          var currentFieldType = $(event.currentTarget).attr("data-setting"); // frequency | currency;
          var currentRow = $(event.currentTarget).closest(".field-group");
          var currentRowIndex = currentRow.index();

          var currentCurrencySelector = $(currentRow).find(
            ".currency-selector"
          );
          var currentFrequencySelector = $(currentRow).find(
            ".frequency-selector"
          );

          var prevFreq = currentFrequencySelector.attr("prev");
          var prevCurrency = currentCurrencySelector.attr("prev");
          var currentCurrency = jQuery(currentCurrencySelector).val();
          var currentFrequency = jQuery(currentFrequencySelector).val();

          if (prevFreq) {
            this.variationStore.addFrequencyToAvailable(
              currentCurrency,
              prevFreq
            );
          }

          if (
            prevFreq != currentFrequency &&
            currentFieldType === "frequency" &&
            currentCurrency !== null
          ) {
            let priceRowElement = jQuery(this.ui.repeaterContainer).find(
              ".currency-selector"
            )[currentRowIndex];
            self.variationStore.refreshCurrencyFieldOptions(
              priceRowElement,
              currentFrequency
            );
          }

          if (
            prevCurrency != currentCurrency &&
            currentFieldType === "currency" &&
            currentFrequency !== null &&
            this.FREE_VERSION_MODE <= 0
          ) {
            var frequencyAvailable = this.defaultFrequencyObj;
            if (this.FREE_VERSION_MODE <= 0) {
              frequencyAvailable = this.variationStore.isFrequencyAvailableForCurrency(
                currentCurrency,
                currentFrequency
              );
            }

            currentFrequencySelector.val("");
            // Move to available the old frequency / currency variation and set as used the new variation
            // Add new frequency as used
            if (frequencyAvailable) {
              this.variationStore.addFrequency(
                currentCurrency,
                currentFrequency
              );

              if (this.FREE_VERSION_MODE <= 0) {
                this.variationStore.addFrequencyToAvailable(
                  prevCurrency,
                  currentFrequency
                );
                currentFrequencySelector.val(currentFrequency);
              }
              currentFrequencySelector.val(frequencyAvailable);
            }

            // Update every frequency selector options with the same currency
            jQuery(this.ui.repeaterContainer)
              .find(".frequency-selector")
              .each(function (priceRowIndex, priceRowElement) {
                if (
                  jQuery(priceRowElement)
                    .closest(".field-group")
                    .find(".currency-selector")
                    .val() === currentCurrency
                ) {
                  self.variationStore.refreshFrequencyFieldOptions(
                    priceRowElement,
                    currentCurrency
                  );
                }
              });
            return;
          }

          // Add frequency as a used and refresh the options for all the selectors with the same currency
          if (currentFrequency !== null && currentCurrency !== null) {
            this.variationStore.addFrequency(currentCurrency, currentFrequency);
            // TODO: avoid duplicate this
            jQuery(this.ui.repeaterContainer)
              .find(".frequency-selector")
              .each(function (priceRowIndex, priceRowElement) {
                if (
                  jQuery(priceRowElement)
                    .closest(".field-group")
                    .find(".currency-selector")
                    .val() === currentCurrency
                ) {
                  self.variationStore.refreshFrequencyFieldOptions(
                    priceRowElement,
                    currentCurrency
                  );
                }
              });
          }

          // Get only frequencies available for the selected currency
          if (currentCurrency && currentFrequency === null) {
            self.variationStore.refreshFrequencyFieldOptions(
              currentFrequencySelector,
              currentCurrency
            );
          }

          // Get only currencies available for the selected frequency
          if (currentFrequency && currentCurrency === null) {
            let priceRowElement = jQuery(this.ui.repeaterContainer).find(
              ".currency-selector"
            )[currentRowIndex];
            self.variationStore.refreshCurrencyFieldOptions(
              priceRowElement,
              currentFrequency
            );
          }

          var self = this;
          var values = this.getControlValue();
          var updated = false;
          var pricingPlan = {
            price: 0,
            currency: currentCurrency,
          };
          if (this.FREE_VERSION_MODE <= 0) {
            pricingPlan.frequency = {
              value: currentFrequency,
              label: currentFrequency,
            };
          }
          let state = values.map((value, index) => {
            if (index === currentRowIndex) {
              updated = true;
              return pricingPlan;
            } else {
              return value;
            }
          });
          if (!updated) state.push(pricingPlan);
          this.onApplyClicked(event);
          //this.setValue(state);
        },
        onButtonRemoveRowClick: function onButtonRemoveRowClick(event) {
          event.preventDefault();
          var self = this;
          var currentRow = $(event.currentTarget).closest(".field-group");
          var currentRowIndex = $(currentRow).index();
          var frequency = currentRow.find(".frequency-selector").val();
          var frequencyLabel = currentRow
            .find(".frequency-selector")
            .children(":selected")
            .text();
          var currency = currentRow.find(".currency-selector").val();

          currentRow.remove();
          var values = this.getControlValue();

          var state = values.filter(function (value, index) {
            if (index !== currentRowIndex) return value;
          });
          this.setValue(state);
          this.variationStore.addFrequencyToAvailable(currency, frequency);

          jQuery(this.ui.repeaterContainer)
            .find(".frequency-selector")
            .each(function (priceRowIndex, priceRowElement) {
              if (
                jQuery(priceRowElement)
                  .closest(".field-group")
                  .find(".currency-selector")
                  .val() === currency
              ) {
                self.variationStore.refreshFrequencyFieldOptions(
                  priceRowElement,
                  currency
                );
              }
            });
        },
        onFieldChange: function onFieldChange(event) {
          this.onApplyClicked(event);
        },
        clearRow: function () {
          this.ui.repeaterContainer.empty();
        },
        appendNewRow: function (rowData, rowIndex) {
          var self = this;
          var totalRows = this.ui.repeaterContainer.children().length;
          var newEmptyRow = false;
          var currency = rowData.currency || "";
          var frequency = rowData.frequency || null;
          var currentRowIndex = rowIndex | (totalRows + 1);

          if (frequency !== null && currency !== null) {
            this.variationStore.addFrequency(currency, frequency.value);
          }
          var frequenciesAvailable;
          if (currency === "") {
            newEmptyRow = true;
            frequenciesAvailable = this.frequencies;
          } else {
            frequenciesAvailable = this.variationStore.getNonUsedVariationsUsedByCurrency(
              currency
            );
          }

          var row = $("<div />", {
            class: "field-group",
          });

          var removeBtn = $(
            '<i class="eicon-close remove-price-row" aria-hidden="true"></i>'
          );

          if (this.FREE_VERSION_MODE <= 0) {
            var frequencyContainer = this.createFrequencyContainer(
              frequency,
              currentRowIndex,
              frequenciesAvailable
            );
            row.prepend(frequencyContainer);
          }

          /** Prepare all fields containers */
          var priceContainer = this.createPriceContainer({
            price: rowData.price || "",
          });
          var currencyContainer = this.createCurrencyContainer(currency);
          row.prepend(currencyContainer).prepend(priceContainer);
          if (totalRows >= 1) {
            row.append(removeBtn);
          }

          this.ui.repeaterContainer.append(row);
          if (!newEmptyRow) {
            jQuery(this.ui.repeaterContainer)
              .find(".frequency-selector")
              .each(function (priceRowIndex, priceRowElement) {
                self.variationStore.refreshFrequencyFieldOptions(
                  priceRowElement,
                  currency
                );
              });
          }
        },
        createFrequencyContainer: function (
          frequencySelected,
          currentRowIndex,
          frequenciesAvailable
        ) {
          var frequencyContainer = $("<div/>");
          var frequencyLabel = $("<label>Subscription frequency</label>");
          var frequencySelect = $("<select />", {
            class: "field frequency-selector",
          });
          frequencySelect.attr("data-setting", "frequency");
          frequenciesAvailable.forEach(function (frequency) {
            frequencySelect.append(
              $("<option>").val(frequency.value).text(frequency.label)
            );
          });
          frequencyLabel.append(frequencySelect);
          frequencyContainer.append(frequencyLabel);
          frequencySelect.val("");

          if (frequencySelected && frequencySelected.value != "") {
            frequencySelect.append(
              $("<option>")
                .val(frequencySelected.value)
                .text(frequencySelected.label)
            );
            frequencySelect.val(frequencySelected.value);
          }
          return frequencyContainer;
        },
        createCurrencyContainer: function (currencySelected) {
          var availableCurrency = this.currencies;

          var currencyContainer = $("<div/>");
          var currencyLabel = $("<label>Currency</label>");
          var currencySelect = $("<select />", {
            class: "field currency-selector",
          });
          currencySelect.attr("data-setting", "currency");
          currencyLabel.append(currencySelect);
          currencyContainer.append(currencyLabel);
          availableCurrency.forEach(function (curreny) {
            currencySelect.append($("<option>").val(curreny).text(curreny));
          });
          currencySelect.val("");
          if (currencySelected != "") currencySelect.val(currencySelected);
          return currencyContainer;
        },
        createPriceContainer: function (data) {
          /** Prepare all fields containers */
          // Price Field
          var priceContainer = $("<div/>");
          var priceLabel = $("<label>Price</label>");
          var price = $("<input />", {
            value: data.price || "",
            class: "field price-input", // you need to quote "class" since it's a reserved keyword
            type: "number",
          });
          price.attr("data-setting", "price");
          priceLabel.append(price);
          priceContainer.append(priceLabel);

          return priceContainer;
        },
        onApplyClicked: function onApplyClicked(event) {
          event.preventDefault();
          var frequencies = this.frequencies;
          var currencies = this.currencies;
          var dataType = $(event.currentTarget).attr("data-setting");
          var dataSelect = $(event.currentTarget).val();
          var currentRow = $(event.currentTarget).closest(".field-group");
          var currentCurrencyVal = $(currentRow)
            .find(".currency-selector")
            .val();
          var currentFrequencyVal = $(currentRow)
            .find(".frequency-selector")
            .val();
          var currentRow = $(event.currentTarget).closest(".field-group");
          var currentRowIndex = $(currentRow).index();
          var currentPriceVal = $(currentRow).find(".price-input").val();
          var currentCurrencyVal = $(currentRow)
            .find(".currency-selector")
            .val();
          var currentFrequencyVal = $(currentRow)
            .find(".frequency-selector")
            .val();
          var updated = false;
          var pricingPlan = {
            price: currentPriceVal,
            currency: currentCurrencyVal,
          };
          if (this.FREE_VERSION_MODE <= 0) {
            pricingPlan.frequency = frequencies.find(
              (frequency) => frequency.value == currentFrequencyVal
            );
          }
          var values = this.getControlValue();
          let state = values.map((value, index) => {
            if (index === currentRowIndex) {
              updated = true;
              return pricingPlan;
            } else {
              return value;
            }
          });
          if (!updated) state.push(pricingPlan);
          this.setValue(state);
        },
      }
    );
    window.elementor.addControlView("pricingplans", pricingplansItemView);
  });
})(jQuery);
