<?php
namespace Paymentpage\includes\elements\PaymentForm;
include('sections/PaymentFormDisplay.php');
include('sections/PricingPlansControls.php');
include('sections/FormControls.php');
include('sections/CurrencySelector.php');
include('sections/SubscriptionSelector.php');
include('sections/ActionsFormSection.php');
include('sections/SubmitButtonControl.php');
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Core\Schemes;
use Paymentpage\includes\elements\PaymentForm\FieldForm;
use Paymentpage\includes\elements\PaymentForm\Controll;

if (! defined('ABSPATH'))
    exit();

class PaymentForm extends \Elementor\Widget_Base{

    private static $freemius;

    public function __construct($data = [], $args = null){
        parent::__construct($data, $args);
        global $payment_page_fs;
        self::$freemius =  $payment_page_fs;
        $this->payment_form_display = new Controll\PaymentFormDisplay($this);
        $this->pricing_plans_controls = new Controll\PricingPlansControls($this);
        $this->form_controls = new Controll\FormControls($this);
        $this->currency_selector = new Controll\CurrencySelector($this);
        $this->subscription_selector = new Controll\SubscriptionSelector($this);
        $this->actions_form_section = new Controll\ActionsFormSection($this);
        $this->submit_button_control = new Controll\SubmitButtonControl($this);
    }

    public function get_name(){
        return 'payment-form';
    }

    public function get_title(){
        return __('Payment Page Form', 'payment-page');
    }

    public function get_icon(){
        return 'eicon-posts-ticker';
    }
    
    public function get_categories(){
        return [
            'payment-page'
        ];
    }

    public function get_script_depends(){
        return [
            'payment-form-component',
            'payment-form'
        ];
    }

    protected function _register_controls(){
        $saved_product_data = get_option('PP_FIRST_PAYMENT_PAGE_DATA');
        $this->payment_form_display->get_controll();
        $this->pricing_plans_controls->get_controll();
        $this->form_controls->get_controll();
        $this->currency_selector->get_controll();
        
        if ( !self::$freemius->is_free_plan() ) {
            $this->subscription_selector->get_controll();
        }

        $this->actions_form_section->get_controll();
        $this->actions_form_section->register_email_action();
        $this->actions_form_section->register_redirect_action();
        $this->actions_form_section->register_dynamic_message_action();
        $this->submit_button_control->get_controll();
        $this->styles_tab();
    }

    /**
     * Start TAB "Styles"
     */
    private function styles_tab(){
        $this->form_controls->styles_controll();
        $this->pricing_plans_controls->styles_settings();
        $this->submit_button_control->styles_settings();
        $this->actions_form_section->styles_settings();
        $this->currency_selector->styles_settings();
    }

    protected function render(){
        global $post;
        $Stripe = new \Paymentpage\includes\controllers\StripeController();
        $stripe_config = $Stripe::get_config();
        $settings = $this->get_settings_for_display();
        $form_styles = $this->form_controls->styles_settings($settings);
        
        $plans = array();
        foreach ($settings['plans'] as $key => $plan ) {
            
            foreach($plan['prices'] as &$price){
                if( !isset($price['frequency']) ||  $price['frequency'] === 'none' ){
                    $price['frequency'] = array('value'=> 'one-time', 'label'=>'One-time');
                }
            }

            $prices = $plan['prices'];
            if ( self::$freemius->is_free_plan() ) {
                $prices = array($prices[0]);
            }

            $plans[] = array(
                'label' => $plan['plan_title'],
                'prices' => $prices,
                'id' => $plan['_id'],
                'post_id' => $post->ID
            );
        }        

        $formFieldBorderRadius = $settings['form_fields_border_radius']['size'] . "" . $settings['form_fields_border_radius']['unit'];

        // // Adding all the font families to the front end published component
        $user_fonts_selected = array(
            $settings['form_field_label_font_family'],
            $settings['form_field_input_font_family'],
            $settings['form_field_input_placeholder_font_family'],
            $settings['submit_button_text_font_family']
        );

        // Avoid to repeat loading of font family
        $google_fonts = array();
        foreach ($user_fonts_selected as $font) {
            if (! in_array($font, $google_fonts)) {
                $google_fonts[] = $font;
            }
        }

        /**
         * Prepare settings JSON from array.
         * WARNING. json_encode will faill on font-family names because some string contain single quotes
         */


        $submit_button_shadow = array(
            "position" => $settings['submit_button_box_shadow_position'] === "inset " ? $settings['submit_button_box_shadow_position'] : "" ,
            "horizontal"=> $settings['submit_button_box_shadow_horizontal']['size'] . $settings['submit_button_box_shadow_horizontal']['unit'],
            "vertical"=> $settings['submit_button_box_shadow_vertical']['size'] . $settings['submit_button_box_shadow_vertical']['unit'],
            "blur"=> $settings['submit_button_box_shadow_blur']['size'] . $settings['submit_button_box_shadow_blur']['unit'],
            "spread"=> $settings['submit_button_box_shadow_spread']['size'] . $settings['submit_button_box_shadow_spread']['unit'],
            "color"=> $settings['submit_button_box_shadow_color'],
        );

        $styleSettings = array(
            'pricingPlanSelector' => array(
                'text' => array(
                    'color' => $settings['pricing_plan_select_text_color']
                ),
                'border' => array(
                    'radius' => $settings['pricing_plan_select_border_radius']['size'] . $settings['pricing_plan_select_border_radius']['unit'],
                    'color' => $settings['pricing_plan_select_border_color'],
                    'size' => $settings['pricing_plan_select_border_size']['size'] . $settings['pricing_plan_select_border_size']['unit']
                ),
                'background' => array(
                    'color' => $settings['pricing_plan_select_background_color']
                ),
                'dropDown' => array(
                    'color' => $settings['pricing_plan_select_dropdown_color'],
                    'background' => $settings['pricing_plan_select_dropdown_background_color'],
                    'borderRadius' => $settings['pricing_plan_select_dropdown_radius']['size'] . $settings['pricing_plan_select_dropdown_radius']['unit']
                ),
                'sectionTitle' => array(
                    'color'=> $settings['pricing_plan_select_title_color'],
                    'fontWeight'=> $settings['pricing_plan_select_title_font_weight'],
                    'fontSize'=> $settings['pricing_plan_select_title_font_size']['size'] . $settings['pricing_plan_select_title_font_size']['unit'] ,
                    'fontFamily' => str_replace([
                        ' ',
                        '\''
                    ], [
                        '_',
                        '\"'
                    ], $settings['pricing_plan_select_title_font_family']),
                    'textTransform'=> $settings['pricing_plan_select_title_font_transform'],
                )
            ),
            'formText' => $form_styles,
            'submitButton' => array(
                'color' => $settings['submit_button_text_color'],
                'background' => $settings['submit_button_background_color'],
                'fontWeight' => $settings['submit_button_text_font_weight'],
                'textTransform' => $settings['submit_button_text_font_transform'],
                'border' => array(
                    'radius' => $settings['submit_button_border_radius']['size'] . $settings['submit_button_border_radius']['unit'],
                    'color' => $settings['submit_button_border_color'],
                    'size' => $settings['submit_button_border_size']['size'] . $settings['submit_button_border_size']['unit']
                ),
                "boxShadow" => $submit_button_shadow['position'] ." ". $submit_button_shadow['horizontal'] ." ". $submit_button_shadow['vertical'] ." ". $submit_button_shadow['blur'] ." ". $submit_button_shadow['spread'] ." ". $submit_button_shadow['color'],
                "padding" => $settings['submit_button_padding']['size'] . $settings['submit_button_padding']['unit'],
                'fontFamily' => str_replace([
                    ' ',
                    '\''
                ], [
                    '_',
                    '\"'
                ], $settings['submit_button_text_font_family'])
            ),
            'formFields' => array(
                'background' => $settings['form_fields_background_color'],
                'border' => array(
                    'color' => $settings['form_fields_border_color'],
                    'size' => $settings['form_fields_border_size']['size'] . $settings['form_fields_border_size']['unit'],
                    'radius' => $formFieldBorderRadius
                ),
                'sectionTitle' => array(
                    'color'=> $settings['form_section_title_style_color'],
                    'fontWeight'=> $settings['form_section_title_style_font_weight'],
                    'fontSize'=> $settings['form_section_title_style_font_size']['size'] . $settings['form_section_title_style_font_size']['unit'] ,
                    'fontFamily' => str_replace([
                        ' ',
                        '\''
                    ], [
                        '_',
                        '\"'
                    ], $settings['form_section_title_style_typography']),
                    'textTransform'=> $settings['form_section_title_style_font_transform'],
                )
            ),
            'currencySelector' => array(
                'display' => ($settings['currency_selector'] === 'no' ? 'none' : 'block')
            ),
            'switchers' =>  array(
                'font'=> array(
                    'fontWeight'=> $settings['switcher_text_font_weight'],
                    'textTransform'=> $settings['switcher_text_font_transform'],
                    'fontSize'=> $settings['switcher_text_font_size']['size'] . $settings['switcher_text_font_size']['unit'],
                    'fontFamily' => str_replace([
                        ' ',
                        '\''
                    ], [
                        '_',
                        '\"'
                    ], $settings['switcher_text_font_family'])
                ),
                'backgroundColor'=> $settings['switcher_button_background_color'],
                'activeBackgroundColor'=> $settings['switcher_button_active_background_color'],
                'padding' => $settings['switcher_button_padding']['size'] . $settings['switcher_button_padding']['unit'],
                'border_initial'=> array(
                    'radius'=> $settings['switcher_button_border_radius']['size'] . $settings['switcher_button_border_radius']['unit'] ,
					'color'=> $settings['switcher_button_border_color'],
					'width'=> $settings['switcher_button_border_size']['size'] . $settings['switcher_button_border_size']['unit'] 
                ),
                'border_active'=> array(
                    'radius'=> $settings['switcher_button_active_border_radius']['size'] . $settings['switcher_button_active_border_radius']['unit'] ,
					'color'=> $settings['switcher_button_active_border_color'],
					'width'=> $settings['switcher_button_active_border_size']['size'] . $settings['switcher_button_active_border_size']['unit']
                ),
            ),
        );
        
        $settingsField = json_encode($styleSettings);

        /**
         * As we are not using the normal Elementor control group for typography, we need to enqueue the google font script for each selected font-family.
         * Elementos do that using JS for the published page.
         */

        foreach ($google_fonts as &$font) {
            $font = str_replace(' ', '+', $font) . ':100,100italic,200,200italic,300,300italic,400,400italic,500,500italic,600,600italic,700,700italic,800,800italic,900,900italic';
        }
        
        $fonts_url = sprintf('https://fonts.googleapis.com/css?family=%s', implode( '&', $google_fonts));
        
        wp_enqueue_style('payment-page-widget-google-forms', $fonts_url);
        
        
        $submitButtonData = array(
            "sorted" => $settings['sorted_text_button']['sorted'],
            "customText1" => str_replace('\'', '', $settings['sorted_text_button']['payment_button_text_1'])  | " ",
            "customText2" => $settings['sorted_text_button']['payment_button_text_2'] | " ",
            "customText3" => $settings['sorted_text_button']['payment_button_text_3'] | " ",
        );

        $pricingSelectorLabel = array(
            "customText" => $settings['pricing_selector_label']['customText'],
            "separatorText" => $settings['pricing_selector_label']['separatorText'],
            "sorted" => $settings['pricing_selector_label']['sorted']
        );
        $stripe_mode = $stripe_config['mode'];
        $account_id = $stripe_config['connection_state'][$stripe_mode]['account_id'];        
        $storeBaseUrl = get_bloginfo('wpurl');
        $currencySymbol = ($settings['currency_symbol'] === "yes" ? "true" : "false"); 
        $public_key = $stripe_mode === "test" ? 'pk_test_SENDylTxyJqxOIHs1t5AuMku00N0hdzhkt' : 'pk_live_MIb9PWRVSzuWUfi9K7bFchL500bPuWlB8q'; 
        echo "<payment-form 
        stripePublicKey='".$public_key."'
        redirectUrl='". ( isset($settings['redirect_to_url']) ?  $settings['redirect_to_url']['url']: "" )."'
        useCurrencySymbol='".$currencySymbol."'
        isFreeVersion='".$settings['FREE_VERSION_MODE']."'
        planSelectorSectionLabel='".$settings['pricing_selector_section_label']."'
        formSectionLabel='".$settings['form_data_section_label']."'
		storeBaseUrl='" . $storeBaseUrl . "'
		stripeConnectedAccount='" . $account_id . "'
		useCurrencySelector='" . ($settings['currency_selector'] === 'yes' ? 'true' : 'false') . "'
		useFrecuencySelector='" . (isset($settings['subscription_selector']) && $settings['subscription_selector'] === '' ? 'false' : 'true') . "'
		pricingSelectorLabel='" . json_encode($pricingSelectorLabel) . "'  
		submitButtonData='" . json_encode($submitButtonData) . "' 
		isOnEdit='false' 
		inlineForm='" . $settings["display_form"] . "' 
		styleSettings='" . json_encode($styleSettings) . "'   
		formFields='" . json_encode($settings['form_fields']) . "' 
		plans='" . json_encode($plans) . "'></payment-form>";
    }

    protected function _content_template(){
        ?>
		<#
		// TODO: be careful with ESMA6/7 code. Needs to be polyfilled or change to conventional JS funcitons. AKA: map Fn. 
		if ( settings.plans.length ) { 
                var defaultFrequency = { value: "one-time", label: "One-time"};
				var plans = settings.plans.map( function( item){
					var plan = {}
					plan.label = item.plan_title.replace(/ /g, "_").trim();
                    plan.id = item._id;
					plan.prices = [];

					plan.prices = item.prices.map( price => {
                        if( !settings.FREE_VERSION_MODE && price.currency !== '' ) {
                            if(price.frequency && price.frequency.value != ''){
                                if( price.frequency == "none" || price.frequency === null ){
                                    price.frequency = defaultFrequency
                                }
                                
                                if( price.frequency.label ){
                                    price.frequency.label = price.frequency.label.replace(/ /g, "_").trim();
                                }
                                return price;
                            }
                            if( !price.frequency ){
                                // There is not frequency. Set it as one time;
                                price.frequency = defaultFrequency
                                return price;
                            }
                        } 
                        if( settings.FREE_VERSION_MODE && price.currency !== '' ){
                            if( !price.frequency ){
                                price.frequency = defaultFrequency
                                    return price;   
                            }else{
                                price.frequency.label = price.frequency.label.replace(/ /g, "_").trim();
                                return price;
                            }

                        }


					})
                    if( settings.FREE_VERSION_MODE && plan.prices.length > 1  ){
                        plan.prices.splice(1);
                    }
					return plan;
					} )
		}
		// Entered text needs to be trimmed, whitesapces changed to underscore.
		// On react side texts should be normalized.
		var formFields = settings.form_fields.map( function( field ){
		return (
		{
			placeholder: field.placeholder.replace(/ /g, "_").trim(),
			label: field.label.replace(/ /g, "_").trim(),
			name: field.name.replace(/ /g, "_").trim(),
		}
		)})

		/**
		*   Prepare the React side expected setting object.
		*   into the  App.js 
        */

        var submitButtonBoxShadow = {
            position: settings.submit_button_box_shadow_position === 'inset' ? settings.submit_button_box_shadow_position : '',
            horizontal: settings.submit_button_box_shadow_horizontal.size + settings.submit_button_box_shadow_horizontal.unit,
            vertical: settings.submit_button_box_shadow_vertical.size + settings.submit_button_box_shadow_vertical.unit,
            blur: settings.submit_button_box_shadow_blur.size + settings.submit_button_box_shadow_blur.unit,
            spread: settings.submit_button_box_shadow_spread.size + settings.submit_button_box_shadow_spread.unit,
            color: settings.submit_button_box_shadow_color,
        };

		var styleSettings = {
			pricingPlanSelector:{
				text:{
					color: settings.pricing_plan_select_text_color
				},
				border:{
					radius: settings.pricing_plan_select_border_radius.size + settings.pricing_plan_select_border_radius.unit ,
					color: settings.pricing_plan_select_border_color,
					size: settings.pricing_plan_select_border_size.size + settings.pricing_plan_select_border_size.unit 
				},
				background:{
					color: settings.pricing_plan_select_background_color
				},
				dropDown:{
					color: settings.pricing_plan_select_dropdown_color ,
					background: settings.pricing_plan_select_dropdown_background_color, // transparent
					borderRadius: settings.pricing_plan_select_dropdown_radius.size +  settings.pricing_plan_select_dropdown_radius.unit
                },
                sectionTitle:{
                    color: settings.pricing_plan_select_title_color ,
					fontWeight: settings.pricing_plan_select_title_font_weight,
                    fontSize: settings.pricing_plan_select_title_font_size.size +  settings.pricing_plan_select_title_font_size.unit,
                    textTransform: settings.pricing_plan_select_title_font_transform,
                    fontFamily: settings.pricing_plan_select_title_font_family.replace(/ /g, "_").trim(),
                }       
			},
			formText:{
				label : {
					color : settings.form_field_label_color,
					fontWeight: settings.form_field_label_font_weight,
					textTransform: settings.form_field_label_font_transform,
					fontSize: settings.form_field_label_font_size.size + settings.form_field_label_font_size.unit,
                    fontFamily: settings.form_field_label_font_family.replace(/ /g, "_").trim(),
                    display: (settings.form_field_label_display === 'yes' ? 'block' : 'none' ),
				},
				input : {
					color : settings.form_field_input_color,
					fontWeight: settings.form_field_input_font_weight,
					textTransform: settings.form_field_input_font_transform,
					fontSize: settings.form_field_input_font_size.size + settings.form_field_input_font_size.unit,
					fontFamily: settings.form_field_input_font_family.replace(/ /g, "_").trim()

				},
				placeholder : {
					color : settings.form_field_input_placeholder_color,
					fontWeight: settings.form_field_input_placeholder_font_weight,
					textTransform: settings.form_field_input_placeholder_font_transform,
					fontSize: settings.form_field_input_placeholder_font_size.size + settings.form_field_input_placeholder_font_size.unit,
					fontFamily: settings.form_field_input_placeholder_font_family.replace(/ /g, "_").trim()

				}
            },
            switchers: {
                font: {
                    fontWeight: settings.switcher_text_font_weight,
                    textTransform: settings.switcher_text_font_transform,
                    fontFamily: settings.switcher_text_font_family.replace(/ /g, "_").trim(),
                    fontSize: settings.switcher_text_font_size.size + settings.switcher_text_font_size.unit,
                },
                backgroundColor: settings.switcher_button_background_color,
                activeBackgroundColor: settings.switcher_button_active_background_color,
                padding : settings.switcher_button_padding.size + settings.switcher_button_padding.unit,
                border_initial: {
					width: settings.switcher_button_border_size.size + settings.switcher_button_border_size.unit,
                    radius: settings.switcher_button_border_radius.size + settings.switcher_button_border_radius.unit,
					color: settings.switcher_button_border_color,
                },
                border_active: {
					width: settings.switcher_button_active_border_size.size + settings.switcher_button_active_border_size.unit,
                    radius: settings.switcher_button_active_border_radius.size + settings.switcher_button_active_border_radius.unit ,
					color: settings.switcher_button_active_border_color,
                },
            },
			submitButton:{
				color : settings.submit_button_text_color,
				background : settings.submit_button_background_color,
				fontWeight : settings.submit_button_text_font_weight,
				textTransform : settings.submit_button_text_font_transform,
				fontSize: settings.submit_button_text_font_size.size + settings.submit_button_text_font_size.unit,
                fontFamily: settings.submit_button_text_font_family.replace(/ /g, "_").trim(),
                border:{
					color: settings.submit_button_border_color,
					width: settings.submit_button_border_size.size + settings.submit_button_border_size.unit,
					radius: settings.submit_button_border_radius.size + settings.submit_button_border_radius.unit 
                },
                boxShadow: submitButtonBoxShadow.position+" "+submitButtonBoxShadow.horizontal+" "+submitButtonBoxShadow.vertical+" "+submitButtonBoxShadow.blur+" "+submitButtonBoxShadow.spread+" "+submitButtonBoxShadow.color ,
                padding: settings.submit_button_padding.size + settings.submit_button_padding.unit,
			},
			formFields:{
				background: settings.form_fields_background_color,
				border:{
					color: settings.form_fields_border_color,
					size: settings.form_fields_border_size.size + settings.form_fields_border_size.unit,
					radius: settings.form_fields_border_radius.size + settings.form_fields_border_radius.unit 
                },
                sectionTitle:{
                    color: settings.form_section_title_style_color ,
					fontWeight: settings.form_section_title_style_font_weight,
                    fontSize: settings.form_section_title_style_font_size.size +  settings.form_section_title_style_font_size.unit,
                    textTransform: settings.form_section_title_style_font_transform,
                    fontFamily: settings.form_section_title_style_font_family.replace(/ /g, "_").trim(),
                }
			},
			currencySelector:{
				display: (settings.currency_selector === "" ? "none" : 'block'),
			}
		};


	
		// Is on edit property help to react component to show/hide some attributes
		// Submit button data
		var sortedButtonText = settings.sorted_text_button.sorted;
		var submitButtonData = {
            sorted: sortedButtonText,
            customText1:  settings.sorted_text_button.payment_button_text_1 && settings.sorted_text_button.payment_button_text_1.replace(/ /g, "_").trim(),
            customText2:  settings.sorted_text_button.payment_button_text_2 && settings.sorted_text_button.payment_button_text_2.replace(/ /g, "_").trim(),
            customText3:  settings.sorted_text_button.payment_button_text_3 && settings.sorted_text_button.payment_button_text_3.replace(/ /g, "_").trim(),
  	    }

		// Select field data to display
		var pricingSelectorLabel = {
			customText: settings.pricing_selector_label.customText.replace(/ /g, "_").trim(),
	    separatorText: settings.pricing_selector_label.separatorText.replace(/ /g, "_").trim(),
  	  sorted: settings.pricing_selector_label.sorted
		}
		var useCurrencySelector = (settings.currency_selector === "yes" ? "true" : "false")
		var useFrequencySelector = (settings.subscription_selector && settings.subscription_selector === "yes" ? "true" : "false")

		var messages = {
			success: settings.success_message.replace(/ /g, "_").trim(),
			error: settings.failure_message.replace(/ /g, "_").trim(),
        }
        
        var  planSelectorSectionLabel = settings.pricing_selector_section_label.replace(/ /g, "_").trim() || "_";
        var formSectionLabel = settings.form_data_section_label.replace(/ /g, "_").trim() || "_";
        var useCurrencySymbol = (settings.currency_symbol === "yes" ? "true" : "false");
        var redirectUrl = (settings.redirect_to_url ? settings.redirect_to_url.url.replace(/ /g, "_").trim() : "_");
        var isOnEdit = "true";
        var isFreeVersion = settings.FREE_VERSION_MODE ? 1 : 0 
		#>
        <payment-form 
            redirectUrl={{redirectUrl}}
            isOnEdit={{isOnEdit}}
            isFreeVersion={{{isFreeVersion}}}
            messages={{JSON.stringify(messages)}}
            planSelectorSectionLabel={{planSelectorSectionLabel}}
            formSectionLabel={{{formSectionLabel}}}
            useCurrencySymbol="{{useCurrencySymbol}}"
            useCurrencySelector="{{useCurrencySelector}}"
            useFrecuencySelector="{{useFrequencySelector}}"
            pricingSelectorLabel={{JSON.stringify(pricingSelectorLabel)}}
            submitButtonData="{{JSON.stringify(submitButtonData)}}"
            inlineForm="{{{settings.display_form}}}"
            styleSettings="{{JSON.stringify(styleSettings)}}"
            formFields={{JSON.stringify(formFields)}} 
            plans={{JSON.stringify(plans)}}></payment-form>
        <?php
        ;
    }
}