<?php 
namespace Paymentpage\includes\elements\PaymentForm\Controll;
use Elementor\Controls_Manager;
use Elementor\Core\Schemes;
use Paymentpage\includes;

class ActionsFormSection  {
  

    public function __construct($elementor){
        $this->elementor = $elementor;
        $this->elementorHelper = new \Paymentpage\includes\helpers\ElementorHelper();
	}

    public function get_controll(){
        $this->elementor->start_controls_section('section_integration', [
        'label' => __('Actions after submit', 'elementor-pro')
        ]);

        $actions = array(
            array(
                'name' => 'email',
                'label' => 'email'
            ),
            array(
                'name' => 'redirect_to',
                'label' => 'Redirect'
            ),
            array(
                'name' => 'dynamic_message',
                'label' => 'Dynamic message'
            )
        );

        $actions_options = array();
        foreach ($actions as $action) {

            $actions_options[$action['name']] = $action['label'];
        }

        $this->elementor->add_control('submit_actions', [
            'label' => __('Add Action', 'elementor-pro'),
            'type' => Controls_Manager::SELECT2,
            'multiple' => true,
            'options' => $actions_options,
            'render_type' => 'none',
            'label_block' => true,
            'default' => [
                'email'
            ],
            'description' => __('Add actions that will be performed after a visitor submits the form (e.g. send an email notification). Choosing an action will add its setting below.', 'elementor-pro')
        ]);

        $this->elementor->end_controls_section();
    }

    public function styles_settings(){
		$this->elementor->start_controls_section('section_dynamic_messasge_style', [
            'label' => __('Dynamic Message', 'payment-page'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);
        $this->dynamic_message_style();
		$this->elementor->end_controls_section();

    }
    private function dynamic_message_style(){

        $defaults = array(
            'font_family'=> "'Open Sans', sans-serif",
            'font_size'=> array(
                'unit' => 'px',
                'size' => 12
            ),
            'font_weight'=> 'normal',
            'font_transform'=> 'none',
        );

        $this->elementorHelper->get_color_control($this->elementor, "" , "dynamic_message_success_text", 'Success Message Color');		
        $this->elementorHelper->get_color_control($this->elementor, "" , "dynamic_message_error_text_color",  'Error Message Color');		
        $this->elementorHelper->get_color_control($this->elementor, "" , "dynamic_message_inline_text_color",  'Inline Message Color');		
        $this->elementorHelper->add_font_styles($this->elementor, "" , 'dynamic_message_label',  $defaults);		
    }

    public function register_email_action(){
        $this->elementor->start_controls_section('section_email', [
        'label' => 'Email',
        'tab' => Controls_Manager::TAB_CONTENT,
        'condition' => [
            'submit_actions' => 'email'
        ]
        ]);

        $this->elementor->add_control('email_to', [
            'label' => __('To', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default' => get_option('admin_email'),
            'placeholder' => get_option('admin_email'),
            'label_block' => true,
            'title' => __('Separate emails with commas', 'elementor-pro'),
            'render_type' => 'none'
        ]);

        /* translators: %s: Site title. */
        $default_message = sprintf(__('New message from "%s"', 'elementor-pro'), get_option('blogname'));

        $this->elementor->add_control('email_subject', [
            'label' => __('Subject', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default' => $default_message,
            'placeholder' => $default_message,
            'label_block' => true,
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('email_content', [
            'label' => __('Message', 'elementor-pro'),
            'type' => Controls_Manager::TEXTAREA,
            'default' => '[all-fields]',
            'placeholder' => '[all-fields]',
            'description' => sprintf(__('By default, all form fields are sent via %s shortcode. To customize sent fields, copy the shortcode that appears inside each field and paste it above.', 'elementor-pro'), '<code>[all-fields]</code>'),
            'render_type' => 'none'
        ]);

        // TODO: create util
        // $site_domain = Utils::get_site_domain();
        $site_domain = 'domain.com';
        $this->elementor->add_control('email_from', [
            'label' => __('From email', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default' => 'email@' . $site_domain,
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('email_from_name', [
            'label' => __('From name', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default' => get_bloginfo('name'),
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('email_reply_to', [
            'label' => __('Reply-To', 'elementor-pro'),
            'type' => Controls_Manager::SELECT,
            'options' => [
                '' => ''
            ],
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('email_to_cc', [
            'label' => __('Cc', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default' => '',
            'title' => __('Separate emails with commas', 'elementor-pro'),
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('email_to_bcc', [
            'label' => __('Bcc', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default' => '',
            'title' => __('Separate emails with commas', 'elementor-pro'),
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('form_metadata', [
            'label' => __('Meta data', 'elementor-pro'),
            'type' => Controls_Manager::SELECT2,
            'multiple' => true,
            'label_block' => true,
            'separator' => 'before',
            'default' => [
                'date',
                'time',
                'page_url',
                'user_agent',
                'remote_ip',
                'credit'
            ],
            'options' => [
                'date' => __('Date', 'elementor-pro'),
                'time' => __('Time', 'elementor-pro'),
                'page_url' => __('Page URL', 'elementor-pro'),
                'user_agent' => __('User Agent', 'elementor-pro'),
                'remote_ip' => __('Remote IP', 'elementor-pro'),
                'credit' => __('Credit', 'elementor-pro')
            ],
            'render_type' => 'none'
        ]);

        $this->elementor->add_control('email_content_type', [
            'label' => __('Send as', 'elementor-pro'),
            'type' => Controls_Manager::SELECT,
            'default' => 'html',
            'render_type' => 'none',
            'options' => [
                'html' => __('HTML', 'elementor-pro'),
                'plain' => __('Plain', 'elementor-pro')
            ]
        ]);

        $this->elementor->end_controls_section();
    }
    public function register_redirect_action(){
        $this->elementor->start_controls_section('section_redirect_to', [
            'label' => __('Redirect', 'payment-page'),
            'condition' => [
                'submit_actions' => 'redirect_to'
            ]
        ]);

        $this->elementor->add_control('redirect_to_url', [
            'label' => __('Redirect to', 'payment-page'),
            'type' => \Elementor\Controls_Manager::URL,
            'placeholder' => __('Type your redirect URL here', 'payment-page')
        ]);

        $this->elementor->end_controls_section();
    }
    public function register_dynamic_message_action(){
        $this->elementor->start_controls_section('dynamic_message', [
            'label' => 'Dynamic message',
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'submit_actions' => 'dynamic_message'
            ]
        ]);

        // Payment success message:
        $this->elementor->add_control('success_message', [
            'label' => __('Payment success message:', 'elementor-pro'),
            'type' => Controls_Manager::TEXTAREA,
            'default'=> 'Your payment has been received. Thank you for your business',
            'label_block' => true,
            'render_type' => 'none'
        ]);

        // Payment failure message:
        $this->elementor->add_control('failure_message', [
            'label' => __('Payment failure message:', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'default'=> 'Payment failed. Please try again',
            'label_block' => true,
            'render_type' => 'none'
        ]);

        // Allow multiple payment attempts?
        $this->elementor->add_control('multiple_attempts', [
            'label' => __('Allow multiple payment attempts? ', 'elementor-pro'),
            'type' => \Elementor\Controls_Manager::SWITCHER,
            'label_on' => __('On', 'payment-page'),
            'label_off' => __('Off', 'payment-page'),
            'default' => 'no'
        ]);

        // Text for link to re-attempt payment:
        $this->elementor->add_control('text_multiple_attempts_link', [
            'label' => __('Payment failure message:', 'elementor-pro'),
            'type' => Controls_Manager::TEXT,
            'label_block' => true,
            'render_type' => 'none',
            'condition' => array(
                'multiple_attempts' => 'yes'
            )
        ]);

        $this->elementor->end_controls_section();
    }
}