<?php 
namespace Paymentpage\includes\elements\PaymentForm\Controll;
use Elementor\Controls_Manager;
use Elementor\Core\Schemes;
use Paymentpage\includes;


class FormControls  {
  

	public function __construct($elementor){
        $this->elementor = $elementor;
        $this->elementorHelper = new \Paymentpage\includes\helpers\ElementorHelper();
        $this->control_name = 'form';
	}

	public function get_controll(){
		$this->elementor->start_controls_section($this->control_name, [
		    'label' => __('Form fields', 'payment-page')
		]);
	
		$this->elementor->add_control(
            $this->control_name."_data_section_label",
            [
            'label' => __( 'Section title', 'payment-page' ),
            'type' => \Elementor\Controls_Manager::TEXT,
            'default' => __( 'PAYMENT METHOD', 'payment-page' ),
            'placeholder' => __( 'Type your section title', 'payment-page' ),
            ]
		);

        $repeater = new \Elementor\Repeater();
        
		$repeater->add_control("label", [
			'label' => __('Label', 'payment-page'),
			'type' => \Elementor\Controls_Manager::TEXT,
			'default' => __('Label', 'payment-page'),
            'placeholder' => __('', 'payment-page')
		]);

		$repeater->add_control("placeholder", [
			'label' => __('Placeholder', 'payment-page'),
			'type' => \Elementor\Controls_Manager::TEXT,
			'default' => __('Placeholder', 'payment-page'),
			'placeholder' => __('', 'payment-page')
		]);

		$repeater->add_control("name", [
			'label' => __('Field name', 'payment-page'),
			'type' => \Elementor\Controls_Manager::HIDDEN,
			'default' => __('Field Name', 'payment-page')
		]);

		$repeater->end_controls_tab();

		$repeater->start_controls_tab($this->control_name.'_fields_advanced_tab', [
			'label' => __('Advanced', 'elementor-pro'),
			'condition' => [
				'field_type!' => 'html'
			]
		]);

		$this->elementor->add_control($this->control_name."_fields", [
			'type' => \Elementor\Controls_Manager::REPEATER,
			'fields' => $repeater->get_controls(),
			'default' => [
				[
					'label' => __('Card number', 'payment-page'),
					'placeholder' => __('Card number', 'payment-page'),
					'name' => __('cardNumber', 'payment-page')
				],
				[
					'label' => __('Expiration date', 'payment-page'),
					'placeholder' => __('Expiration date', 'payment-page'),
					'name' => __('expiration', 'payment-page')
				],
				[
					'label' => __('CVC', 'payment-page'),
					'placeholder' => __('CVC', 'payment-page'),
					'name' => __('cvc', 'payment-page')
				],
				[
					'label' => __('ZIP', 'payment-page'),
					'placeholder' => __('ZIP', 'payment-page'),
					'name' => __('zip', 'payment-page')
				],
				[
					'label' => __('First name', 'payment-page'),
					'placeholder' => __('First name', 'payment-page'),
					'name' => __('firstName', 'payment-page')
				],
				[
					'label' => __('Last name', 'payment-page'),
					'placeholder' => __('Last name', 'payment-page'),
					'name' => __('lastName', 'payment-page')
				],
				[
					'label' => __('Email address', 'payment-page'),
					'placeholder' => __('Email address', 'payment-page'),
					'name' => __('email', 'payment-page')
				]
			],
			'title_field' => '{{{ label }}}',
			'item_actions' => [
				'add' => false,
				'duplicate' => false,
				'remove' => false,
				'sort' => false
			]
		]);

		$this->elementor->end_controls_section();
	}

  	public function styles_controll(){
		$this->elementor->start_controls_section( $this->control_name.'_section_title_style', [
            'label' => __('Form', 'payment-page'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        
        $this->field_label_style();
		$this->form_fields_style();
		$this->input_styles();
        $this->placeholder_styles();
        $this->section_label();

		$this->elementor->end_controls_section();

	}
    
    private function section_label(){
        // Section title
        $field_name = "section_title";
        $this->elementor->add_control( $this->control_name. '_section_title_style_title', [
            'label' => __('Section title', 'payment-page'),
            'type' => Controls_Manager::HEADING,
            'separator' => 'before'
        ]);

        $this->elementor->add_control ( $this->control_name. '_section_title_style_color', [
            'label' => __('Title color', 'payment-page'),
            'type' => Controls_Manager::COLOR,
            'default' => '#2676f1',
            'scheme' => [
                'type' => Schemes\Color::get_type(),
                'value' => Schemes\Color::COLOR_1
            ]
        ]);
        $this->elementor->add_control( $this->control_name. "_section_title_style_typography", [
            'label' => __('Typography', 'payment-page'),
            'type' => \Elementor\Controls_Manager::POPOVER_TOGGLE,
            'label_off' => __('Default', 'payment-page'),
            'label_on' => __('Custom', 'payment-page'),
            'return_value' => 'yes'
        ]);
        $this->elementor->start_popover();
        $this->elementor->add_control( $this->control_name. "_section_title_style_font_family", [
            'label' => __('Font Family', 'payment-page'),
            'type' => \Elementor\Controls_Manager::FONT,
            'default' => "'Open Sans', sans-serif"
        ]);
        $this->elementor->add_control( $this->control_name. "_section_title_style_font_size", [
            'label' => __('Size', 'payment-page'),
            'type' => Controls_Manager::SLIDER,
            'size_units' => [
                'px'
            ],
            'default' => array(
                'unit' => 'px',
                'size' => 12
            ),
            'range' => [
                'px' => [
                    'min' => 1,
                    'max' => 200
                ]
            ],
            'responsive' => true
        ]);

        $typo_weight_options = array(
            '' => __('Default', 'elementor')
        );

        foreach (array_merge([
            'normal',
            'bold'
        ], range(100, 900, 100)) as $weight) {
            $typo_weight_options[$weight] = ucfirst($weight);
        }

        $this->elementor->add_control( $this->control_name. "_section_title_style_font_weight", [
            'label' => _x('Weight', 'Typography Control', 'elementor'),
            'type' => Controls_Manager::SELECT,
            'default' => 'normal',
            'options' => $typo_weight_options
        ]);

        $this->elementor->add_control( $this->control_name. "_section_title_style_font_transform", [
            'label' => _x('Transform', 'Typography Control', 'elementor'),
            'type' => Controls_Manager::SELECT,
            'default' => 'none',
            'options' => [
                '' => __('Default', 'elementor'),
                'uppercase' => __('Uppercase', 'elementor'),
                'lowercase' => _x('Lowercase', 'Typography Control', 'elementor'),
                'capitalize' => _x('Capitalize', 'Typography Control', 'elementor'),
                'none' => _x('Normal', 'Typography Control', 'elementor')
            ]
        ]);

    }
    
	private function field_label_style(){
        $field_name = "field_label";
		$this->elementor->add_control( $this->control_name. '_'.$field_name.'_styles_heading', [
			'label' => __('Field labels', 'payment-page'),
			'type' => Controls_Manager::HEADING,
			'separator' => 'before'
        ]);
        
        $this->elementorHelper->get_siwtch_control($this->elementor, $this->control_name, $field_name );
        $this->elementorHelper->get_color_control($this->elementor, $this->control_name , $field_name);		
        $this->elementorHelper->add_font_styles($this->elementor, $this->control_name,  $field_name);
	}

	private function input_styles(){
        $field_name = 'field_input';
        $defaults = array(
            'font_family'=> "'Open Sans', sans-serif",
            'font_size'=> array(
                'unit' => 'px',
                'size' => 12
            ),
            'font_weight'=> 'normal',
            'font_transform'=> 'none',
        );
        $this->elementorHelper->get_color_control($this->elementor, $this->control_name , $field_name, null , '#8676aa');		
        $this->elementorHelper->add_font_styles($this->elementor, $this->control_name , $field_name,  $defaults);		
       
	}
	
	private function placeholder_styles(){
        $field_name = 'field_input_placeholder';
        $defaults = array(
            'font_family'=> "'Open Sans', sans-serif",
            'font_size'=> array(
                'unit' => 'px',
                'size' => 13
            ),
            'font_weight'=> 'normal',
            'font_transform'=> 'none',
        );
        $this->elementor->add_control($this->control_name. '_'.$field_name.'_heading', [
            'label' => __('Placeholder text', 'payment-page'),
            'type' => Controls_Manager::HEADING,
            'separator' => 'before'
        ]);
        $this->elementorHelper->get_siwtch_control($this->elementor, $this->control_name, $field_name, 'yes' );
        $this->elementorHelper->get_color_control($this->elementor, $this->control_name, $field_name, null, '#32325d' );
        $this->elementorHelper->add_font_styles($this->elementor, $this->control_name , $field_name,  $defaults);		
	}
	
	private function form_fields_style(){
        $field_name = 'fields';
		$this->elementor->add_control( $this->control_name. '_'.$field_name.'_style_heading', [
            'label' => __('Field input', 'payment-page'),
            'type' => Controls_Manager::HEADING,
            'separator' => 'before'
        ]);

        $this->elementor->add_control( $this->control_name. '_'.$field_name.'_style', [
            'label' => __('Border', 'payment-page'),
            'type' => Controls_Manager::HEADING,
            'separator' => 'before'
        ]);
        $this->elementorHelper->get_border_control($this->elementor, $this->control_name, $field_name);
        $this->elementor->add_control('hr', [
            'type' => \Elementor\Controls_Manager::DIVIDER
        ]);
        
        $this->elementorHelper->get_background_control($this->elementor, $this->control_name, $field_name);
       
    }

    public function styles_settings($settings){
        $labelFontSize = $settings['form_field_label_font_size']['size'] . "" . $settings['form_field_label_font_size']['unit'];
        $inputFontSize = $settings['form_field_input_font_size']['size'] . "" . $settings['form_field_input_font_size']['unit'];
        $placeholderFontSize = $settings['form_field_input_placeholder_font_size']['size'] . "" . $settings['form_field_input_placeholder_font_size']['unit'];

        return array(
            'label' => array(
                'fontSize' => $labelFontSize,
                'color' => $settings['form_field_label_color'],
                'fontWeight' => $settings['form_field_label_font_weight'],
                'textTransform' => $settings['form_field_label_font_transform'],
                'display' => ( $settings['form_field_label_display'] === 'yes' ? 'block' : 'none' ),
                'fontFamily' => str_replace([
                    ' ',
                    '\''
                ], [
                    '_',
                    '\"'
                ], $settings['form_field_label_font_family'])
            ),
            'input' => array(
                'fontSize' => $inputFontSize,
                'color' => $settings['form_field_input_color'],
                'fontWeight' => $settings['form_field_input_font_weight'],
                'textTransform' => $settings['form_field_input_font_transform'],
                'fontFamily' => str_replace([
                    ' ',
                    '\''
                ], [
                    '_',
                    '\"'
                ], $settings['form_field_input_font_family'])
            ),
            'placeholder' => array(
                'fontSize' => $placeholderFontSize,
                'color' => $settings['form_field_input_placeholder_color'],
                'fontWeight' => $settings['form_field_input_placeholder_font_weight'],
                'textTransform' => $settings['form_field_input_placeholder_font_transform'],
                'fontFamily' => str_replace([
                    ' ',
                    '\''
                ], [
                    '_',
                    '\"'
                ], $settings['form_field_input_placeholder_font_family'])
            )
            );
    }

}