<?php
namespace Paymentpage\includes\elements;

class Widgets
{

    private static $config;

    private static $Stripe_Gateway;

    public function __construct()
    {
        self::$Stripe_Gateway = new \Paymentpage\includes\controllers\StripeController();
        self::$config = self::$Stripe_Gateway::get_config();

        // Register widget scripts
        add_action('elementor/frontend/after_register_scripts', [
            __CLASS__,
            'widget_scripts'
        ]);

        // Register controls
        add_action('elementor/controls/controls_registered', [
            __CLASS__,
            'register_controls'
        ]);

        // Register widgets
        add_action('elementor/widgets/widgets_registered', [
            __CLASS__,
            'register_widgets'
        ]);

        // Register custom category
        add_action('elementor/elements/categories_registered', [
            __CLASS__,
            'register_category'
        ]);
    }

    /*
     * @version: 1.0
     */
    public static function register_controls()
    {
        $controls_manager = \Elementor\Plugin::$instance->controls_manager;
        $controls_manager->register_control('pricingplans', new \Paymentpage\includes\elements\Controlls\Pricing\PricingControll(self::$config));
        $controls_manager->register_control('fieldssorted', new \Paymentpage\includes\elements\Controlls\FieldsSorted\FieldsSortedControll());
        $controls_manager->register_control('pricing_selector_data', new \Paymentpage\includes\elements\Controlls\PricingSelectorData\PricingSelectorControll());
    }

    /**
     * Register Widgets
     *
     * Register new Elementor widgets.
     *
     * @since 1.0
     * @access public
     */
    public static function register_widgets()
    {
        // Its is now safe to include Widgets files

        // Register Widgets0
        \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new \Paymentpage\includes\elements\PaymentForm\PaymentForm());
    }

    public static function widget_scripts()
    {
        wp_register_script('stripe-library', 'https://js.stripe.com/v3/');
        wp_register_script('payment-form-component', plugins_url('components/payment-form/payment-form.js', dirname(__FILE__)));
        wp_register_script('payment-form', plugins_url('PaymentForm/PaymentForm.js', __FILE__), [
            'stripe-library',
            'jQuery'
        ]);
    }

    public static function register_category($elements_manager)
    {
        $elements_manager->add_category('payment-page', [
            'title' => __('Payment Page', 'payment-page'),
            'icon' => 'fa fa-plug'
        ]);
    }
}
