<?php
namespace Paymentpage\includes\helpers;
use Elementor\Controls_Manager;
use Elementor\Core\Schemes;
class ElementorHelper{

  public function get_border_control($elementor, $control_name, $field_name, $defaults=null){
    $elementor->add_control($control_name.'_'.$field_name.'_border_color', [
      'label' => __('Color', 'payment-page'),
      'type' => Controls_Manager::COLOR,
      'default' =>  $defaults['border_color'] ? $defaults['border_color'] : '#cec3e6',
      'scheme' => [
          'type' => Schemes\Color::get_type(),
          'value' => Schemes\Color::COLOR_1
      ]
    ]);

    $elementor->add_control( $control_name.'_'.$field_name.'_border_size', [
        'label' => __('Size', 'payment-page'),
        'type' => Controls_Manager::SLIDER,
        'size_units' => [
            'px'
        ],
        'default' =>  $defaults['border_size'] ? $defaults['border_size'] : array(
            'unit' => 'px',
            'size' => 1
        ),
        'range' => [
            'px' => [
                'min' => 0,
                'max' => 20
            ]
        ],
        'responsive' => true
    ]);

    $elementor->add_control( $control_name.'_'.$field_name.'_border_radius', [
        'label' => __('Radius', 'payment-page'),
        'type' => Controls_Manager::SLIDER,
        'size_units' => [
            'px'
        ],
        'default' => $defaults['border_radius'] ? $defaults['border_radius'] : array(
            'unit' => 'px',
            'size' => 3
        ),
        'range' => [
            'px' => [
                'min' => 1,
                'max' => 200
            ]
        ],
        'responsive' => true
    ]);
  }

  public function get_background_control($elementor, $control_name, $field_name, $field_label=null, $defaults=null){
    $elementor->add_control($control_name. '_'.$field_name.'_background_color', [
      'label' => __( $field_label ? $field_label : 'Background', 'payment-page'),
      'type' => Controls_Manager::COLOR,
      'default' => $defaults ? $defaults : "#ffffff",
      'scheme' => [
          'type' => Schemes\Color::get_type(),
          'value' => Schemes\Color::COLOR_1
      ]
    ]);
  }

  public static function add_font_styles($elementor, $control_name, $field_name, $defaults = null){

    $elementor->add_control($control_name.'_'.$field_name.'_typography', [
      'label' => __('Typography', 'payment-page'),
      'type' => \Elementor\Controls_Manager::POPOVER_TOGGLE,
      'label_off' => __('Default', 'payment-page'),
      'label_on' => __('Custom', 'payment-page'),
      'return_value' => 'yes'
    ]);
    $elementor->start_popover();
    $elementor->add_control($control_name.'_'.$field_name.'_font_family', [
      'label' => __('Font Family', 'payment-page'),
      'type' => \Elementor\Controls_Manager::FONT,
      'default' =>  $defaults['font_family'] ? $defaults['font_family'] :  "'Open Sans', sans-serif"
    ]);
    $elementor->add_control( $control_name.'_'.$field_name.'_font_size', [
      'label' => __('Size', 'payment-page'),
      'type' => \Elementor\Controls_Manager::SLIDER,
      'size_units' => [
        'px'
      ],
      'default' => $defaults['font_size'] ? $defaults['font_size'] : array(
        'unit' => 'px',
        'size' => 15
      ),
      'range' => [
        'px' => [
          'min' => 1,
          'max' => 200
                ],
      ],
      'responsive' => true
    ]);

    $typo_weight_options = array(
      '' => __('Default', 'elementor')
    );

    foreach (array_merge([
      'normal',
      'bold'
    ], range(100, 900, 100)) as $weight) {
      $typo_weight_options[$weight] = ucfirst($weight);
    }

    $elementor->add_control($control_name.'_'.$field_name.'_font_weight', [
      'label' => _x('Weight', 'Typography Control', 'elementor'),
      'type' => \Elementor\Controls_Manager::SELECT,
      'default' => $defaults['font_weight'] ? $defaults['font_weight'] : 'normal',
      'options' => $typo_weight_options
    ]);

    $elementor->add_control( $control_name.'_'.$field_name.'_font_transform', [
      'label' => _x('Transform', 'Typography Control', 'elementor'),
      'type' => \Elementor\Controls_Manager::SELECT,
      'default' =>  $defaults['font_transform'] ? $defaults['font_transform'] :  'none',
      'options' => [
        '' => __('Default', 'elementor'),
        'uppercase' => __('Uppercase', 'elementor'),
        'lowercase' => _x('Lowercase', 'Typography Control', 'elementor'),
        'capitalize' => _x('Capitalize', 'Typography Control', 'elementor'),
        'none' => _x('Normal', 'Typography Control', 'elementor')
      ]
    ]);
      
    $elementor->end_popover();
    return $elementor;
  }

  public static function get_box_shadow($elementor, $control_name, $field_name,$field_label=null, $defaults = null){
    $elementor->add_control($control_name.'_'.$field_name.'_box_shadow', [
      'label' => __('Box Shadow', 'payment-page'),
      'type' => \Elementor\Controls_Manager::POPOVER_TOGGLE,
      'label_off' => __('Default', 'payment-page'),
      'label_on' => __('Custom', 'payment-page'),
      'return_value' => 'yes'
    ]);
    $elementor->start_popover();
    $elementor->add_control($control_name.'_'.$field_name.'_box_shadow_color', [
      'label' => __( $field_label ? $field_label : 'Color', 'elementor'),
      'type' => \Elementor\Controls_Manager::COLOR,
      'default' =>  $defaults ? $defaults :'transparent',
      'scheme' => [
        'type' => \Elementor\Core\Schemes\Color::get_type(),
        'value' => \Elementor\Core\Schemes\Color::COLOR_1
      ]
    ]);

    $box_shadow_properties = ['horizontal', 'vertical', 'blur', 'spread'];
    foreach( $box_shadow_properties as $property ){
      $elementor->add_control( $control_name.'_'.$field_name.'_box_shadow_'.$property, [
        'label' => __( ucwords($property), 'payment-page'),
        'type' => Controls_Manager::SLIDER,
        'size_units' => [
            'px'
        ],
        'default' =>  $defaults ? $defaults : array(
            'unit' => 'px',
            'size' => 0
        ),
        'range' => [
            'px' => [
                'min' => -100,
                'max' => 100
            ]
        ],
        'responsive' => true
      ]);
    }

    $actions_options = array(
      'inset' => 'Inset',
      'outline'=> 'Outline',
    );

    $elementor->add_control($control_name.'_'.$field_name.'_box_shadow_position', [
        'label' => __('Position', 'elementor-pro'),
        'type' => Controls_Manager::SELECT,
        'options' => $actions_options,
        'render_type' => 'none',
        'label_block' => true,
    ]);

    $elementor->end_popover();
    return $elementor;
  }

  public static function get_siwtch_control($elementor, $control_name, $field_name, $defaults=null){
    $elementor->add_control( $control_name.'_'.$field_name.'_display', [
      'label' => __('On/Off', 'payment-page'),
      'type' => \Elementor\Controls_Manager::SWITCHER,
      'label_on' => __('On', 'payment-page'),
      'label_off' => __('Off', 'payment-page'),
      'default' =>  $defaults ? $defaults : 'yes'
    ]);
  }

  public static function get_color_control($elementor, $control_name, $field_name, $field_label=null, $defaults=null){
    return $elementor->add_control($control_name.'_'.$field_name.'_color', [
      'label' => __( $field_label ? $field_label : 'Text Color', 'elementor'),
      'type' => \Elementor\Controls_Manager::COLOR,
      'default' =>  $defaults ? $defaults :'#8676aa',
      'scheme' => [
        'type' => \Elementor\Core\Schemes\Color::get_type(),
        'value' => \Elementor\Core\Schemes\Color::COLOR_1
      ]
        ]);
  }

  public static function get_padding_control($elementor, $control_name, $field_name, $field_label=null, $default=null){
    $elementor->add_control( $control_name.'_'.$field_name.'_padding', [
      'label' => __('Padding', 'payment-page'),
      'type' => Controls_Manager::SLIDER,
      'size_units' => [
          'px'
      ],
      'default' =>  $default ? $default : array(
          'unit' => 'px',
          'size' => 0
      ),
      'range' => [
          'px' => [
              'min' => 0,
              'max' => 20
          ]
      ],
      'responsive' => true
    ]);
  }

  public static function get_section_title($elementor, $control_name, $field_name, $field_label=null){
    $elementor->add_control( $control_name. '_'.$field_name.'_styles_heading', [
      'label' => __( $field_label ? $field_label : 'Field labels', 'payment-page'),
      'type' => Controls_Manager::HEADING,
      'separator' => 'before'
        ]);
    
  }
}